from raypyc.colors import *
from raypyc.enums import *
from raypyc.structures import *


# Initialize window and OpenGL context
def init_window(width: int, height: int, title: bytes) -> None:
	pass

# Check if KEY_ESCAPE pressed or Close icon pressed
def window_should_close() -> bool:
	pass

# Close window and unload OpenGL context
def close_window() -> None:
	pass

# Check if window has been initialized successfully
def is_window_ready() -> bool:
	pass

# Check if window is currently fullscreen
def is_window_fullscreen() -> bool:
	pass

# Check if window is currently hidden (only PLATFORM_DESKTOP)
def is_window_hidden() -> bool:
	pass

# Check if window is currently minimized (only PLATFORM_DESKTOP)
def is_window_minimized() -> bool:
	pass

# Check if window is currently maximized (only PLATFORM_DESKTOP)
def is_window_maximized() -> bool:
	pass

# Check if window is currently focused (only PLATFORM_DESKTOP)
def is_window_focused() -> bool:
	pass

# Check if window has been resized last frame
def is_window_resized() -> bool:
	pass

# Check if one specific window flag is enabled
def is_window_state(flag: int) -> bool:
	pass

# Set window configuration state using flags (only PLATFORM_DESKTOP)
def set_window_state(flags: int) -> None:
	pass

# Clear window configuration state flags
def clear_window_state(flags: int) -> None:
	pass

# Toggle window state: fullscreen/windowed (only PLATFORM_DESKTOP)
def toggle_fullscreen() -> None:
	pass

# Set window state: maximized, if resizable (only PLATFORM_DESKTOP)
def maximize_window() -> None:
	pass

# Set window state: minimized, if resizable (only PLATFORM_DESKTOP)
def minimize_window() -> None:
	pass

# Set window state: not minimized/maximized (only PLATFORM_DESKTOP)
def restore_window() -> None:
	pass

# Set icon for window (only PLATFORM_DESKTOP)
def set_window_icon(image: Image) -> None:
	pass

# Set title for window (only PLATFORM_DESKTOP)
def set_window_title(title: bytes) -> None:
	pass

# Set window position on screen (only PLATFORM_DESKTOP)
def set_window_position(x: int, y: int) -> None:
	pass

# Set monitor for the current window (fullscreen mode)
def set_window_monitor(monitor: int) -> None:
	pass

# Set window minimum dimensions (for FLAG_WINDOW_RESIZABLE)
def set_window_min_size(width: int, height: int) -> None:
	pass

# Set window dimensions
def set_window_size(width: int, height: int) -> None:
	pass

# Set window opacity [0.0f..1.0f] (only PLATFORM_DESKTOP)
def set_window_opacity(opacity: float) -> None:
	pass

# Get native window handle
def get_window_handle() -> int:
	pass

# Get current screen width
def get_screen_width() -> int:
	pass

# Get current screen height
def get_screen_height() -> int:
	pass

# Get current render width (it considers HiDPI)
def get_render_width() -> int:
	pass

# Get current render height (it considers HiDPI)
def get_render_height() -> int:
	pass

# Get number of connected monitors
def get_monitor_count() -> int:
	pass

# Get current connected monitor
def get_current_monitor() -> int:
	pass

# Get specified monitor position
def get_monitor_position(monitor: int) -> Vector2:
	pass

# Get specified monitor width (current video mode used by monitor)
def get_monitor_width(monitor: int) -> int:
	pass

# Get specified monitor height (current video mode used by monitor)
def get_monitor_height(monitor: int) -> int:
	pass

# Get specified monitor physical width in millimetres
def get_monitor_physical_width(monitor: int) -> int:
	pass

# Get specified monitor physical height in millimetres
def get_monitor_physical_height(monitor: int) -> int:
	pass

# Get specified monitor refresh rate
def get_monitor_refresh_rate(monitor: int) -> int:
	pass

# Get window position XY on monitor
def get_window_position() -> Vector2:
	pass

# Get window scale DPI factor
def get_window_scale_dpi() -> Vector2:
	pass

# Get the human-readable, UTF-8 encoded name of the primary monitor
def get_monitor_name(monitor: int) -> bytes:
	pass

# Set clipboard text content
def set_clipboard_text(text: bytes) -> None:
	pass

# Get clipboard text content
def get_clipboard_text() -> bytes:
	pass

# Enable waiting for events on EndDrawing(), no automatic event polling
def enable_event_waiting() -> None:
	pass

# Disable waiting for events on EndDrawing(), automatic events polling
def disable_event_waiting() -> None:
	pass

# Swap back buffer with front buffer (screen drawing)
def swap_screen_buffer() -> None:
	pass

# Register all input events
def poll_input_events() -> None:
	pass

# Wait for some time (halt program execution)
def wait_time(seconds: float) -> None:
	pass

# Shows cursor
def show_cursor() -> None:
	pass

# Hides cursor
def hide_cursor() -> None:
	pass

# Check if cursor is not visible
def is_cursor_hidden() -> bool:
	pass

# Enables cursor (unlock cursor)
def enable_cursor() -> None:
	pass

# Disables cursor (lock cursor)
def disable_cursor() -> None:
	pass

# Check if cursor is on the screen
def is_cursor_on_screen() -> bool:
	pass

# Set background color (framebuffer clear color)
def clear_background(color: Color) -> None:
	pass

# Setup canvas (framebuffer) to start drawing
def begin_drawing() -> None:
	pass

# End canvas drawing and swap buffers (double buffering)
def end_drawing() -> None:
	pass

# Begin 2D mode with custom camera (2D)
def begin_mode_2d(camera: Camera2D) -> None:
	pass

# Ends 2D mode with custom camera
def end_mode_2d() -> None:
	pass

# Begin 3D mode with custom camera (3D)
def begin_mode_3d(camera: Camera3D) -> None:
	pass

# Ends 3D mode and returns to default 2D orthographic mode
def end_mode_3d() -> None:
	pass

# Begin drawing to render texture
def begin_texture_mode(target: RenderTexture2D) -> None:
	pass

# Ends drawing to render texture
def end_texture_mode() -> None:
	pass

# Begin custom shader drawing
def begin_shader_mode(shader: Shader) -> None:
	pass

# End custom shader drawing (use default shader)
def end_shader_mode() -> None:
	pass

# Begin blending mode (alpha, additive, multiplied, subtract, custom)
def begin_blend_mode(mode: int) -> None:
	pass

# End blending mode (reset to default: alpha blending)
def end_blend_mode() -> None:
	pass

# Begin scissor mode (define screen area for following drawing)
def begin_scissor_mode(x: int, y: int, width: int, height: int) -> None:
	pass

# End scissor mode
def end_scissor_mode() -> None:
	pass

# Begin stereo rendering (requires VR simulator)
def begin_vr_stereo_mode(config: VrStereoConfig) -> None:
	pass

# End stereo rendering (requires VR simulator)
def end_vr_stereo_mode() -> None:
	pass

# Load VR stereo config for VR simulator device parameters
def load_vr_stereo_config(device: VrDeviceInfo) -> VrStereoConfig:
	pass

# Unload VR stereo config
def unload_vr_stereo_config(config: VrStereoConfig) -> None:
	pass

# Load shader from files and bind default locations
def load_shader(vsFileName: bytes, fsFileName: bytes) -> Shader:
	pass

# Load shader from code strings and bind default locations
def load_shader_from_memory(vsCode: bytes, fsCode: bytes) -> Shader:
	pass

# Get shader uniform location
def get_shader_location(shader: Shader, uniformName: bytes) -> int:
	pass

# Get shader attribute location
def get_shader_location_attrib(shader: Shader, attribName: bytes) -> int:
	pass

# Set shader uniform value
def set_shader_value(shader: Shader, locIndex: int, value: int, uniformType: int) -> None:
	pass

# Set shader uniform value vector
def set_shader_value_v(shader: Shader, locIndex: int, value: int, uniformType: int, count: int) -> None:
	pass

# Set shader uniform value (matrix 4x4)
def set_shader_value_matrix(shader: Shader, locIndex: int, mat: Matrix) -> None:
	pass

# Set shader uniform value for texture (sampler2d)
def set_shader_value_texture(shader: Shader, locIndex: int, texture: Texture2D) -> None:
	pass

# Unload shader from GPU memory (VRAM)
def unload_shader(shader: Shader) -> None:
	pass

# Get a ray trace from mouse position
def get_mouse_ray(mousePosition: Vector2, camera: Camera) -> Ray:
	pass

# Get camera transform matrix (view matrix)
def get_camera_matrix(camera: Camera) -> Matrix:
	pass

# Get camera 2d transform matrix
def get_camera_matrix_2d(camera: Camera2D) -> Matrix:
	pass

# Get the screen space position for a 3d world space position
def get_world_to_screen(position: Vector3, camera: Camera) -> Vector2:
	pass

# Get the world space position for a 2d camera screen space position
def get_screen_to_world_2d(position: Vector2, camera: Camera2D) -> Vector2:
	pass

# Get size position for a 3d world space position
def get_world_to_screen_ex(position: Vector3, camera: Camera, width: int, height: int) -> Vector2:
	pass

# Get the screen space position for a 2d camera world space position
def get_world_to_screen_2d(position: Vector2, camera: Camera2D) -> Vector2:
	pass

# Set target FPS (maximum)
def set_target_fps(fps: int) -> None:
	pass

# Get current FPS
def get_fps() -> int:
	pass

# Get time in seconds for last frame drawn (delta time)
def get_frame_time() -> float:
	pass

# Get elapsed time in seconds since InitWindow()
def get_time() -> float:
	pass

# Get a random value between min and max (both included)
def get_random_value(min: int, max: int) -> int:
	pass

# Set the seed for the random number generator
def set_random_seed(seed: int) -> None:
	pass

# Takes a screenshot of current screen (filename extension defines format)
def take_screenshot(fileName: bytes) -> None:
	pass

# Setup init configuration flags (view FLAGS)
def set_config_flags(flags: int) -> None:
	pass

# Show trace log messages (LOG_DEBUG, LOG_INFO, LOG_WARNING, LOG_ERROR...)
def trace_log(logLevel: int, text: bytes, args: ...) -> None:
	pass

# Set the current threshold (minimum) log level
def set_trace_log_level(logLevel: int) -> None:
	pass

# Internal memory allocator
def mem_alloc(size: int) -> int:
	pass

# Internal memory reallocator
def mem_realloc(ptr: int, size: int) -> int:
	pass

# Internal memory free
def mem_free(ptr: int) -> None:
	pass

# Open URL with default system browser (if available)
def open_url(url: bytes) -> None:
	pass

# Load file data as byte array (read)
def load_file_data(fileName: bytes, bytesRead: POINTER(c_uint)) -> POINTER(c_ubyte):  # Load file data as byte array (read)
	pass

# Unload file data allocated by LoadFileData()
def unload_file_data(data: POINTER(c_ubyte)) -> None:
	pass

# Save data to file from byte array (write), returns true on success
def save_file_data(fileName: bytes, data: int, bytesToWrite: int) -> bool:
	pass

# Export data to code (.h), returns true on success
def export_data_as_code(data: bytes, size: int, fileName: bytes) -> bool:
	pass

# Load text data from file (read), returns a '\0' terminated string
def load_file_text(fileName: bytes) -> bytes:
	pass

# Unload file text data allocated by LoadFileText()
def unload_file_text(text: bytes) -> None:
	pass

# Save text data to file (write), string must be '\0' terminated, returns true on success
def save_file_text(fileName: bytes, text: bytes) -> bool:
	pass

# Check if file exists
def file_exists(fileName: bytes) -> bool:
	pass

# Check if a directory path exists
def directory_exists(dirPath: bytes) -> bool:
	pass

# Check file extension (including point: .png, .wav)
def is_file_extension(fileName: bytes, ext: bytes) -> bool:
	pass

# Get file length in bytes (NOTE: GetFileSize() conflicts with windows.h)
def get_file_length(fileName: bytes) -> int:
	pass

# Get pointer to extension for a filename string (includes dot: '.png')
def get_file_extension(fileName: bytes) -> bytes:
	pass

# Get pointer to filename for a path string
def get_file_name(filePath: bytes) -> bytes:
	pass

# Get filename string without extension (uses static string)
def get_file_name_without_ext(filePath: bytes) -> bytes:
	pass

# Get full path for a given fileName with path (uses static string)
def get_directory_path(filePath: bytes) -> bytes:
	pass

# Get previous directory path for a given path (uses static string)
def get_prev_directory_path(dirPath: bytes) -> bytes:
	pass

# Get current working directory (uses static string)
def get_working_directory() -> bytes:
	pass

# Get the directory if the running application (uses static string)
def get_application_directory() -> bytes:
	pass

# Change working directory, return true on success
def change_directory(dir: bytes) -> bool:
	pass

# Check if a given path is a file or a directory
def is_path_file(path: bytes) -> bool:
	pass

# Load directory filepaths
def load_directory_files(dirPath: bytes) -> FilePathList:
	pass

# Load directory filepaths with extension filtering and recursive directory scan
def load_directory_files_ex(basePath: bytes, filter: bytes, scanSubdirs: bool) -> FilePathList:
	pass

# Unload filepaths
def unload_directory_files(files: FilePathList) -> None:
	pass

# Check if a file has been dropped into window
def is_file_dropped() -> bool:
	pass

# Load dropped filepaths
def load_dropped_files() -> FilePathList:
	pass

# Unload dropped filepaths
def unload_dropped_files(files: FilePathList) -> None:
	pass

# Get file modification time (last write time)
def get_file_mod_time(fileName: bytes) -> int:
	pass

# Compress data (DEFLATE algorithm), memory must be MemFree()
def compress_data(data: POINTER(c_ubyte), dataSize: int, compDataSize: POINTER(c_int)) -> POINTER(c_ubyte):  # Compress data (DEFLATE algorithm), memory must be MemFree()
	pass

# Decompress data (DEFLATE algorithm), memory must be MemFree()
def decompress_data(compData: POINTER(c_ubyte), compDataSize: int, dataSize: POINTER(c_int)) -> POINTER(c_ubyte):  # Decompress data (DEFLATE algorithm), memory must be MemFree()
	pass

# Encode data to Base64 string, memory must be MemFree()
def encode_data_base64(data: POINTER(c_ubyte), dataSize: int, outputSize: POINTER(c_int)) -> bytes:
	pass

# Decode Base64 string data, memory must be MemFree()
def decode_data_base64(data: POINTER(c_ubyte), outputSize: POINTER(c_int)) -> POINTER(c_ubyte):  # Decode Base64 string data, memory must be MemFree()
	pass

# Check if a key has been pressed once
def is_key_pressed(key: int) -> bool:
	pass

# Check if a key is being pressed
def is_key_down(key: int) -> bool:
	pass

# Check if a key has been released once
def is_key_released(key: int) -> bool:
	pass

# Check if a key is NOT being pressed
def is_key_up(key: int) -> bool:
	pass

# Set a custom key to exit program (default is ESC)
def set_exit_key(key: int) -> None:
	pass

# Get key pressed (keycode), call it multiple times for keys queued, returns 0 when the queue is empty
def get_key_pressed() -> int:
	pass

# Get char pressed (unicode), call it multiple times for chars queued, returns 0 when the queue is empty
def get_char_pressed() -> int:
	pass

# Check if a gamepad is available
def is_gamepad_available(gamepad: int) -> bool:
	pass

# Get gamepad internal name id
def get_gamepad_name(gamepad: int) -> bytes:
	pass

# Check if a gamepad button has been pressed once
def is_gamepad_button_pressed(gamepad: int, button: int) -> bool:
	pass

# Check if a gamepad button is being pressed
def is_gamepad_button_down(gamepad: int, button: int) -> bool:
	pass

# Check if a gamepad button has been released once
def is_gamepad_button_released(gamepad: int, button: int) -> bool:
	pass

# Check if a gamepad button is NOT being pressed
def is_gamepad_button_up(gamepad: int, button: int) -> bool:
	pass

# Get the last gamepad button pressed
def get_gamepad_button_pressed() -> int:
	pass

# Get gamepad axis count for a gamepad
def get_gamepad_axis_count(gamepad: int) -> int:
	pass

# Get axis movement value for a gamepad axis
def get_gamepad_axis_movement(gamepad: int, axis: int) -> float:
	pass

# Set internal gamepad mappings (SDL_GameControllerDB)
def set_gamepad_mappings(mappings: bytes) -> int:
	pass

# Check if a mouse button has been pressed once
def is_mouse_button_pressed(button: int) -> bool:
	pass

# Check if a mouse button is being pressed
def is_mouse_button_down(button: int) -> bool:
	pass

# Check if a mouse button has been released once
def is_mouse_button_released(button: int) -> bool:
	pass

# Check if a mouse button is NOT being pressed
def is_mouse_button_up(button: int) -> bool:
	pass

# Get mouse position X
def get_mouse_x() -> int:
	pass

# Get mouse position Y
def get_mouse_y() -> int:
	pass

# Get mouse position XY
def get_mouse_position() -> Vector2:
	pass

# Get mouse delta between frames
def get_mouse_delta() -> Vector2:
	pass

# Set mouse position XY
def set_mouse_position(x: int, y: int) -> None:
	pass

# Set mouse offset
def set_mouse_offset(offsetX: int, offsetY: int) -> None:
	pass

# Set mouse scaling
def set_mouse_scale(scaleX: float, scaleY: float) -> None:
	pass

# Get mouse wheel movement for X or Y, whichever is larger
def get_mouse_wheel_move() -> float:
	pass

# Get mouse wheel movement for both X and Y
def get_mouse_wheel_move_v() -> Vector2:
	pass

# Set mouse cursor
def set_mouse_cursor(cursor: int) -> None:
	pass

# Get touch position X for touch point 0 (relative to screen size)
def get_touch_x() -> int:
	pass

# Get touch position Y for touch point 0 (relative to screen size)
def get_touch_y() -> int:
	pass

# Get touch position XY for a touch point index (relative to screen size)
def get_touch_position(index: int) -> Vector2:
	pass

# Get touch point identifier for given index
def get_touch_point_id(index: int) -> int:
	pass

# Get number of touch points
def get_touch_point_count() -> int:
	pass

# Enable a set of gestures using flags
def set_gestures_enabled(flags: int) -> None:
	pass

# Check if a gesture have been detected
def is_gesture_detected(gesture: int) -> bool:
	pass

# Get latest detected gesture
def get_gesture_detected() -> int:
	pass

# Get gesture hold time in milliseconds
def get_gesture_hold_duration() -> float:
	pass

# Get gesture drag vector
def get_gesture_drag_vector() -> Vector2:
	pass

# Get gesture drag angle
def get_gesture_drag_angle() -> float:
	pass

# Get gesture pinch delta
def get_gesture_pinch_vector() -> Vector2:
	pass

# Get gesture pinch angle
def get_gesture_pinch_angle() -> float:
	pass

# Set camera mode (multiple camera modes available)
def set_camera_mode(camera: Camera, mode: int) -> None:
	pass

# Update camera position for selected mode
def update_camera(camera: POINTER(Camera)) -> None:
	pass

# Set camera pan key to combine with mouse movement (free camera)
def set_camera_pan_control(keyPan: int) -> None:
	pass

# Set camera alt key to combine with mouse movement (free camera)
def set_camera_alt_control(keyAlt: int) -> None:
	pass

# Set camera smooth zoom key to combine with mouse (free camera)
def set_camera_smooth_zoom_control(keySmoothZoom: int) -> None:
	pass

# Set camera move controls (1st person and 3rd person cameras)
def set_camera_move_controls(keyFront: int, keyBack: int, keyRight: int, keyLeft: int, keyUp: int, keyDown: int) -> None:
	pass

# Set texture and rectangle to be used on shapes drawing
def set_shapes_texture(texture: Texture2D, source: Rectangle) -> None:
	pass

# Draw a pixel
def draw_pixel(posX: int, posY: int, color: Color) -> None:
	pass

# Draw a pixel (Vector version)
def draw_pixel_v(position: Vector2, color: Color) -> None:
	pass

# Draw a line
def draw_line(startPosX: int, startPosY: int, endPosX: int, endPosY: int, color: Color) -> None:
	pass

# Draw a line (Vector version)
def draw_line_v(startPos: Vector2, endPos: Vector2, color: Color) -> None:
	pass

# Draw a line defining thickness
def draw_line_ex(startPos: Vector2, endPos: Vector2, thick: float, color: Color) -> None:
	pass

# Draw a line using cubic-bezier curves in-out
def draw_line_bezier(startPos: Vector2, endPos: Vector2, thick: float, color: Color) -> None:
	pass

# Draw line using quadratic bezier curves with a control point
def draw_line_bezier_quad(startPos: Vector2, endPos: Vector2, controlPos: Vector2, thick: float, color: Color) -> None:
	pass

# Draw line using cubic bezier curves with 2 control points
def draw_line_bezier_cubic(startPos: Vector2, endPos: Vector2, startControlPos: Vector2, endControlPos: Vector2, thick: float, color: Color) -> None:
	pass

# Draw lines sequence
def draw_line_strip(points: POINTER(Vector2), pointCount: int, color: Color) -> None:
	pass

# Draw a color-filled circle
def draw_circle(centerX: int, centerY: int, radius: float, color: Color) -> None:
	pass

# Draw a piece of a circle
def draw_circle_sector(center: Vector2, radius: float, startAngle: float, endAngle: float, segments: int, color: Color) -> None:
	pass

# Draw circle sector outline
def draw_circle_sector_lines(center: Vector2, radius: float, startAngle: float, endAngle: float, segments: int, color: Color) -> None:
	pass

# Draw a gradient-filled circle
def draw_circle_gradient(centerX: int, centerY: int, radius: float, color1: Color, color2: Color) -> None:
	pass

# Draw a color-filled circle (Vector version)
def draw_circle_v(center: Vector2, radius: float, color: Color) -> None:
	pass

# Draw circle outline
def draw_circle_lines(centerX: int, centerY: int, radius: float, color: Color) -> None:
	pass

# Draw ellipse
def draw_ellipse(centerX: int, centerY: int, radiusH: float, radiusV: float, color: Color) -> None:
	pass

# Draw ellipse outline
def draw_ellipse_lines(centerX: int, centerY: int, radiusH: float, radiusV: float, color: Color) -> None:
	pass

# Draw ring
def draw_ring(center: Vector2, innerRadius: float, outerRadius: float, startAngle: float, endAngle: float, segments: int, color: Color) -> None:
	pass

# Draw ring outline
def draw_ring_lines(center: Vector2, innerRadius: float, outerRadius: float, startAngle: float, endAngle: float, segments: int, color: Color) -> None:
	pass

# Draw a color-filled rectangle
def draw_rectangle(posX: int, posY: int, width: int, height: int, color: Color) -> None:
	pass

# Draw a color-filled rectangle (Vector version)
def draw_rectangle_v(position: Vector2, size: Vector2, color: Color) -> None:
	pass

# Draw a color-filled rectangle
def draw_rectangle_rec(rec: Rectangle, color: Color) -> None:
	pass

# Draw a color-filled rectangle with pro parameters
def draw_rectangle_pro(rec: Rectangle, origin: Vector2, rotation: float, color: Color) -> None:
	pass

# Draw a vertical-gradient-filled rectangle
def draw_rectangle_gradient_v(posX: int, posY: int, width: int, height: int, color1: Color, color2: Color) -> None:
	pass

# Draw a horizontal-gradient-filled rectangle
def draw_rectangle_gradient_h(posX: int, posY: int, width: int, height: int, color1: Color, color2: Color) -> None:
	pass

# Draw a gradient-filled rectangle with custom vertex colors
def draw_rectangle_gradient_ex(rec: Rectangle, col1: Color, col2: Color, col3: Color, col4: Color) -> None:
	pass

# Draw rectangle outline
def draw_rectangle_lines(posX: int, posY: int, width: int, height: int, color: Color) -> None:
	pass

# Draw rectangle outline with extended parameters
def draw_rectangle_lines_ex(rec: Rectangle, lineThick: float, color: Color) -> None:
	pass

# Draw rectangle with rounded edges
def draw_rectangle_rounded(rec: Rectangle, roundness: float, segments: int, color: Color) -> None:
	pass

# Draw rectangle with rounded edges outline
def draw_rectangle_rounded_lines(rec: Rectangle, roundness: float, segments: int, lineThick: float, color: Color) -> None:
	pass

# Draw a color-filled triangle (vertex in counter-clockwise order!)
def draw_triangle(v1: Vector2, v2: Vector2, v3: Vector2, color: Color) -> None:
	pass

# Draw triangle outline (vertex in counter-clockwise order!)
def draw_triangle_lines(v1: Vector2, v2: Vector2, v3: Vector2, color: Color) -> None:
	pass

# Draw a triangle fan defined by points (first vertex is the center)
def draw_triangle_fan(points: POINTER(Vector2), pointCount: int, color: Color) -> None:
	pass

# Draw a triangle strip defined by points
def draw_triangle_strip(points: POINTER(Vector2), pointCount: int, color: Color) -> None:
	pass

# Draw a regular polygon (Vector version)
def draw_poly(center: Vector2, sides: int, radius: float, rotation: float, color: Color) -> None:
	pass

# Draw a polygon outline of n sides
def draw_poly_lines(center: Vector2, sides: int, radius: float, rotation: float, color: Color) -> None:
	pass

# Draw a polygon outline of n sides with extended parameters
def draw_poly_lines_ex(center: Vector2, sides: int, radius: float, rotation: float, lineThick: float, color: Color) -> None:
	pass

# Check collision between two rectangles
def check_collision_recs(rec1: Rectangle, rec2: Rectangle) -> bool:
	pass

# Check collision between two circles
def check_collision_circles(center1: Vector2, radius1: float, center2: Vector2, radius2: float) -> bool:
	pass

# Check collision between circle and rectangle
def check_collision_circle_rec(center: Vector2, radius: float, rec: Rectangle) -> bool:
	pass

# Check if point is inside rectangle
def check_collision_point_rec(point: Vector2, rec: Rectangle) -> bool:
	pass

# Check if point is inside circle
def check_collision_point_circle(point: Vector2, center: Vector2, radius: float) -> bool:
	pass

# Check if point is inside a triangle
def check_collision_point_triangle(point: Vector2, p1: Vector2, p2: Vector2, p3: Vector2) -> bool:
	pass

# Check the collision between two lines defined by two points each, returns collision point by reference
def check_collision_lines(startPos1: Vector2, endPos1: Vector2, startPos2: Vector2, endPos2: Vector2, collisionPoint: POINTER(Vector2)) -> bool:
	pass

# Check if point belongs to line created between two points [p1] and [p2] with defined margin in pixels [threshold]
def check_collision_point_line(point: Vector2, p1: Vector2, p2: Vector2, threshold: int) -> bool:
	pass

# Get collision rectangle for two rectangles collision
def get_collision_rec(rec1: Rectangle, rec2: Rectangle) -> Rectangle:
	pass

# Load image from file into CPU memory (RAM)
def load_image(fileName: bytes) -> Image:
	pass

# Load image from RAW file data
def load_image_raw(fileName: bytes, width: int, height: int, format: int, headerSize: int) -> Image:
	pass

# Load image sequence from file (frames appended to image.data)
def load_image_anim(fileName: bytes, frames: POINTER(c_int)) -> Image:
	pass

# Load image from memory buffer, fileType refers to extension: i.e. '.png'
def load_image_from_memory(fileType: bytes, fileData: POINTER(c_ubyte), dataSize: int) -> Image:
	pass

# Load image from GPU texture data
def load_image_from_texture(texture: Texture2D) -> Image:
	pass

# Load image from screen buffer and (screenshot)
def load_image_from_screen() -> Image:
	pass

# Unload image from CPU memory (RAM)
def unload_image(image: Image) -> None:
	pass

# Export image data to file, returns true on success
def export_image(image: Image, fileName: bytes) -> bool:
	pass

# Export image as code file defining an array of bytes, returns true on success
def export_image_as_code(image: Image, fileName: bytes) -> bool:
	pass

# Generate image: plain color
def gen_image_color(width: int, height: int, color: Color) -> Image:
	pass

# Generate image: vertical gradient
def gen_image_gradient_v(width: int, height: int, top: Color, bottom: Color) -> Image:
	pass

# Generate image: horizontal gradient
def gen_image_gradient_h(width: int, height: int, left: Color, right: Color) -> Image:
	pass

# Generate image: radial gradient
def gen_image_gradient_radial(width: int, height: int, density: float, inner: Color, outer: Color) -> Image:
	pass

# Generate image: checked
def gen_image_checked(width: int, height: int, checksX: int, checksY: int, col1: Color, col2: Color) -> Image:
	pass

# Generate image: white noise
def gen_image_white_noise(width: int, height: int, factor: float) -> Image:
	pass

# Generate image: cellular algorithm, bigger tileSize means bigger cells
def gen_image_cellular(width: int, height: int, tileSize: int) -> Image:
	pass

# Create an image duplicate (useful for transformations)
def image_copy(image: Image) -> Image:
	pass

# Create an image from another image piece
def image_from_image(image: Image, rec: Rectangle) -> Image:
	pass

# Create an image from text (default font)
def image_text(text: bytes, fontSize: int, color: Color) -> Image:
	pass

# Create an image from text (custom sprite font)
def image_text_ex(font: Font, text: bytes, fontSize: float, spacing: float, tint: Color) -> Image:
	pass

# Convert image data to desired format
def image_format(image: POINTER(Image), newFormat: int) -> None:
	pass

# Convert image to POT (power-of-two)
def image_to_pot(image: POINTER(Image), fill: Color) -> None:
	pass

# Crop an image to a defined rectangle
def image_crop(image: POINTER(Image), crop: Rectangle) -> None:
	pass

# Crop image depending on alpha value
def image_alpha_crop(image: POINTER(Image), threshold: float) -> None:
	pass

# Clear alpha channel to desired color
def image_alpha_clear(image: POINTER(Image), color: Color, threshold: float) -> None:
	pass

# Apply alpha mask to image
def image_alpha_mask(image: POINTER(Image), alphaMask: Image) -> None:
	pass

# Premultiply alpha channel
def image_alpha_premultiply(image: POINTER(Image)) -> None:
	pass

# Resize image (Bicubic scaling algorithm)
def image_resize(image: POINTER(Image), newWidth: int, newHeight: int) -> None:
	pass

# Resize image (Nearest-Neighbor scaling algorithm)
def image_resize_nn(image: POINTER(Image), newWidth: int, newHeight: int) -> None:
	pass

# Resize canvas and fill with color
def image_resize_canvas(image: POINTER(Image), newWidth: int, newHeight: int, offsetX: int, offsetY: int, fill: Color) -> None:
	pass

# Compute all mipmap levels for a provided image
def image_mipmaps(image: POINTER(Image)) -> None:
	pass

# Dither image data to 16bpp or lower (Floyd-Steinberg dithering)
def image_dither(image: POINTER(Image), rBpp: int, gBpp: int, bBpp: int, aBpp: int) -> None:
	pass

# Flip image vertically
def image_flip_vertical(image: POINTER(Image)) -> None:
	pass

# Flip image horizontally
def image_flip_horizontal(image: POINTER(Image)) -> None:
	pass

# Rotate image clockwise 90deg
def image_rotate_cw(image: POINTER(Image)) -> None:
	pass

# Rotate image counter-clockwise 90deg
def image_rotate_ccw(image: POINTER(Image)) -> None:
	pass

# Modify image color: tint
def image_color_tint(image: POINTER(Image), color: Color) -> None:
	pass

# Modify image color: invert
def image_color_invert(image: POINTER(Image)) -> None:
	pass

# Modify image color: grayscale
def image_color_grayscale(image: POINTER(Image)) -> None:
	pass

# Modify image color: contrast (-100 to 100)
def image_color_contrast(image: POINTER(Image), contrast: float) -> None:
	pass

# Modify image color: brightness (-255 to 255)
def image_color_brightness(image: POINTER(Image), brightness: int) -> None:
	pass

# Modify image color: replace color
def image_color_replace(image: POINTER(Image), color: Color, replace: Color) -> None:
	pass

# Load color data from image as a Color array (RGBA - 32bit)
def load_image_colors(image: Image) -> POINTER(Color):  # Load color data from image as a Color array (RGBA - 32bit)
	pass

# Load colors palette from image as a Color array (RGBA - 32bit)
def load_image_palette(image: Image, maxPaletteSize: int, colorCount: POINTER(c_int)) -> POINTER(Color):  # Load colors palette from image as a Color array (RGBA - 32bit)
	pass

# Unload color data loaded with LoadImageColors()
def unload_image_colors(colors: POINTER(Color)) -> None:
	pass

# Unload colors palette loaded with LoadImagePalette()
def unload_image_palette(colors: POINTER(Color)) -> None:
	pass

# Get image alpha border rectangle
def get_image_alpha_border(image: Image, threshold: float) -> Rectangle:
	pass

# Get image pixel color at (x, y) position
def get_image_color(image: Image, x: int, y: int) -> Color:
	pass

# Clear image background with given color
def image_clear_background(dst: POINTER(Image), color: Color) -> None:
	pass

# Draw pixel within an image
def image_draw_pixel(dst: POINTER(Image), posX: int, posY: int, color: Color) -> None:
	pass

# Draw pixel within an image (Vector version)
def image_draw_pixel_v(dst: POINTER(Image), position: Vector2, color: Color) -> None:
	pass

# Draw line within an image
def image_draw_line(dst: POINTER(Image), startPosX: int, startPosY: int, endPosX: int, endPosY: int, color: Color) -> None:
	pass

# Draw line within an image (Vector version)
def image_draw_line_v(dst: POINTER(Image), start: Vector2, end: Vector2, color: Color) -> None:
	pass

# Draw circle within an image
def image_draw_circle(dst: POINTER(Image), centerX: int, centerY: int, radius: int, color: Color) -> None:
	pass

# Draw circle within an image (Vector version)
def image_draw_circle_v(dst: POINTER(Image), center: Vector2, radius: int, color: Color) -> None:
	pass

# Draw rectangle within an image
def image_draw_rectangle(dst: POINTER(Image), posX: int, posY: int, width: int, height: int, color: Color) -> None:
	pass

# Draw rectangle within an image (Vector version)
def image_draw_rectangle_v(dst: POINTER(Image), position: Vector2, size: Vector2, color: Color) -> None:
	pass

# Draw rectangle within an image
def image_draw_rectangle_rec(dst: POINTER(Image), rec: Rectangle, color: Color) -> None:
	pass

# Draw rectangle lines within an image
def image_draw_rectangle_lines(dst: POINTER(Image), rec: Rectangle, thick: int, color: Color) -> None:
	pass

# Draw a source image within a destination image (tint applied to source)
def image_draw(dst: POINTER(Image), src: Image, srcRec: Rectangle, dstRec: Rectangle, tint: Color) -> None:
	pass

# Draw text (using default font) within an image (destination)
def image_draw_text(dst: POINTER(Image), text: bytes, posX: int, posY: int, fontSize: int, color: Color) -> None:
	pass

# Draw text (custom sprite font) within an image (destination)
def image_draw_text_ex(dst: POINTER(Image), font: Font, text: bytes, position: Vector2, fontSize: float, spacing: float, tint: Color) -> None:
	pass

# Load texture from file into GPU memory (VRAM)
def load_texture(fileName: bytes) -> Texture2D:
	pass

# Load texture from image data
def load_texture_from_image(image: Image) -> Texture2D:
	pass

# Load cubemap from image, multiple image cubemap layouts supported
def load_texture_cubemap(image: Image, layout: int) -> TextureCubemap:
	pass

# Load texture for rendering (framebuffer)
def load_render_texture(width: int, height: int) -> RenderTexture2D:
	pass

# Unload texture from GPU memory (VRAM)
def unload_texture(texture: Texture2D) -> None:
	pass

# Unload render texture from GPU memory (VRAM)
def unload_render_texture(target: RenderTexture2D) -> None:
	pass

# Update GPU texture with new data
def update_texture(texture: Texture2D, pixels: int) -> None:
	pass

# Update GPU texture rectangle with new data
def update_texture_rec(texture: Texture2D, rec: Rectangle, pixels: int) -> None:
	pass

# Generate GPU mipmaps for a texture
def gen_texture_mipmaps(texture: POINTER(Texture2D)) -> None:
	pass

# Set texture scaling filter mode
def set_texture_filter(texture: Texture2D, filter: int) -> None:
	pass

# Set texture wrapping mode
def set_texture_wrap(texture: Texture2D, wrap: int) -> None:
	pass

# Draw a Texture2D
def draw_texture(texture: Texture2D, posX: int, posY: int, tint: Color) -> None:
	pass

# Draw a Texture2D with position defined as Vector2
def draw_texture_v(texture: Texture2D, position: Vector2, tint: Color) -> None:
	pass

# Draw a Texture2D with extended parameters
def draw_texture_ex(texture: Texture2D, position: Vector2, rotation: float, scale: float, tint: Color) -> None:
	pass

# Draw a part of a texture defined by a rectangle
def draw_texture_rec(texture: Texture2D, source: Rectangle, position: Vector2, tint: Color) -> None:
	pass

# Draw texture quad with tiling and offset parameters
def draw_texture_quad(texture: Texture2D, tiling: Vector2, offset: Vector2, quad: Rectangle, tint: Color) -> None:
	pass

# Draw part of a texture (defined by a rectangle) with rotation and scale tiled into dest.
def draw_texture_tiled(texture: Texture2D, source: Rectangle, dest: Rectangle, origin: Vector2, rotation: float, scale: float, tint: Color) -> None:
	pass

# Draw a part of a texture defined by a rectangle with 'pro' parameters
def draw_texture_pro(texture: Texture2D, source: Rectangle, dest: Rectangle, origin: Vector2, rotation: float, tint: Color) -> None:
	pass

# Draws a texture (or part of it) that stretches or shrinks nicely
def draw_texture_n_patch(texture: Texture2D, nPatchInfo: NPatchInfo, dest: Rectangle, origin: Vector2, rotation: float, tint: Color) -> None:
	pass

# Draw a textured polygon
def draw_texture_poly(texture: Texture2D, center: Vector2, points: POINTER(Vector2), texcoords: POINTER(Vector2), pointCount: int, tint: Color) -> None:
	pass

# Get color with alpha applied, alpha goes from 0.0f to 1.0f
def fade(color: Color, alpha: float) -> Color:
	pass

# Get hexadecimal value for a Color
def color_to_int(color: Color) -> int:
	pass

# Get Color normalized as float [0..1]
def color_normalize(color: Color) -> Vector4:
	pass

# Get Color from normalized values [0..1]
def color_from_normalized(normalized: Vector4) -> Color:
	pass

# Get HSV values for a Color, hue [0..360], saturation/value [0..1]
def color_to_hsv(color: Color) -> Vector3:
	pass

# Get a Color from HSV values, hue [0..360], saturation/value [0..1]
def color_from_hsv(hue: float, saturation: float, value: float) -> Color:
	pass

# Get color with alpha applied, alpha goes from 0.0f to 1.0f
def color_alpha(color: Color, alpha: float) -> Color:
	pass

# Get src alpha-blended into dst color with tint
def color_alpha_blend(dst: Color, src: Color, tint: Color) -> Color:
	pass

# Get Color structure from hexadecimal value
def get_color(hexValue: int) -> Color:
	pass

# Get Color from a source pixel pointer of certain format
def get_pixel_color(srcPtr: int, format: int) -> Color:
	pass

# Set color formatted into destination pixel pointer
def set_pixel_color(dstPtr: int, color: Color, format: int) -> None:
	pass

# Get pixel data size in bytes for certain format
def get_pixel_data_size(width: int, height: int, format: int) -> int:
	pass

# Get the default Font
def get_font_default() -> Font:
	pass

# Load font from file into GPU memory (VRAM)
def load_font(fileName: bytes) -> Font:
	pass

# Load font from file with extended parameters, use NULL for fontChars and 0 for glyphCount to load the default character set
def load_font_ex(fileName: bytes, fontSize: int, fontChars: POINTER(c_int), glyphCount: int) -> Font:
	pass

# Load font from Image (XNA style)
def load_font_from_image(image: Image, key: Color, firstChar: int) -> Font:
	pass

# Load font from memory buffer, fileType refers to extension: i.e. '.ttf'
def load_font_from_memory(fileType: bytes, fileData: POINTER(c_ubyte), dataSize: int, fontSize: int, fontChars: POINTER(c_int), glyphCount: int) -> Font:
	pass

# Load font data for further use
def load_font_data(fileData: POINTER(c_ubyte), dataSize: int, fontSize: int, fontChars: POINTER(c_int), glyphCount: int, type: int) -> POINTER(GlyphInfo):  # Load font data for further use
	pass

# Generate image font atlas using chars info
def gen_image_font_atlas(chars: POINTER(GlyphInfo), recs: POINTER(POINTER(Rectangle)), glyphCount: int, fontSize: int, padding: int, packMethod: int) -> Image:
	pass

# Unload font chars info data (RAM)
def unload_font_data(chars: POINTER(GlyphInfo), glyphCount: int) -> None:
	pass

# Unload font from GPU memory (VRAM)
def unload_font(font: Font) -> None:
	pass

# Export font as code file, returns true on success
def export_font_as_code(font: Font, fileName: bytes) -> bool:
	pass

# Draw current FPS
def draw_fps(posX: int, posY: int) -> None:
	pass

# Draw text (using default font)
def draw_text(text: bytes, posX: int, posY: int, fontSize: int, color: Color) -> None:
	pass

# Draw text using font and additional parameters
def draw_text_ex(font: Font, text: bytes, position: Vector2, fontSize: float, spacing: float, tint: Color) -> None:
	pass

# Draw text using Font and pro parameters (rotation)
def draw_text_pro(font: Font, text: bytes, position: Vector2, origin: Vector2, rotation: float, fontSize: float, spacing: float, tint: Color) -> None:
	pass

# Draw one character (codepoint)
def draw_text_codepoint(font: Font, codepoint: int, position: Vector2, fontSize: float, tint: Color) -> None:
	pass

# Draw multiple character (codepoint)
def draw_text_codepoints(font: Font, codepoints: POINTER(c_int), count: int, position: Vector2, fontSize: float, spacing: float, tint: Color) -> None:
	pass

# Measure string width for default font
def measure_text(text: bytes, fontSize: int) -> int:
	pass

# Measure string size for Font
def measure_text_ex(font: Font, text: bytes, fontSize: float, spacing: float) -> Vector2:
	pass

# Get glyph index position in font for a codepoint (unicode character), fallback to '?' if not found
def get_glyph_index(font: Font, codepoint: int) -> int:
	pass

# Get glyph font info data for a codepoint (unicode character), fallback to '?' if not found
def get_glyph_info(font: Font, codepoint: int) -> GlyphInfo:
	pass

# Get glyph rectangle in font atlas for a codepoint (unicode character), fallback to '?' if not found
def get_glyph_atlas_rec(font: Font, codepoint: int) -> Rectangle:
	pass

# Load all codepoints from a UTF-8 text string, codepoints count returned by parameter
def load_codepoints(text: bytes, count: POINTER(c_int)) -> POINTER(c_int):  # Load all codepoints from a UTF-8 text string, codepoints count returned by parameter
	pass

# Unload codepoints data from memory
def unload_codepoints(codepoints: POINTER(c_int)) -> None:
	pass

# Get total number of codepoints in a UTF-8 encoded string
def get_codepoint_count(text: bytes) -> int:
	pass

# Get next codepoint in a UTF-8 encoded string, 0x3f('?') is returned on failure
def get_codepoint(text: bytes, bytesProcessed: POINTER(c_int)) -> int:
	pass

# Encode one codepoint into UTF-8 byte array (array length returned as parameter)
def codepoint_to_utf8(codepoint: int, byteSize: POINTER(c_int)) -> bytes:
	pass

# Encode text as codepoints array into UTF-8 text string (WARNING: memory must be freed!)
def text_codepoints_to_utf8(codepoints: POINTER(c_int), length: int) -> bytes:
	pass

# Copy one string to another, returns bytes copied
def text_copy(dst: bytes, src: bytes) -> int:
	pass

# Check if two text string are equal
def text_is_equal(text1: bytes, text2: bytes) -> bool:
	pass

# Get text length, checks for '\0' ending
def text_length(text: bytes) -> int:
	pass

# Text formatting with variables (sprintf() style)
def text_format(text: bytes, args: ...) -> bytes:
	pass

# Get a piece of a text string
def text_subtext(text: bytes, position: int, length: int) -> bytes:
	pass

# Replace text string (WARNING: memory must be freed!)
def text_replace(text: bytes, replace: bytes, by: bytes) -> bytes:
	pass

# Insert text in a position (WARNING: memory must be freed!)
def text_insert(text: bytes, insert: bytes, position: int) -> bytes:
	pass

# Join text strings with delimiter
def text_join(textList: POINTER(POINTER(c_char)), count: int, delimiter: bytes) -> bytes:
	pass

# Split text into multiple strings
def text_split(text: bytes, delimiter: bytes, count: POINTER(c_int)) -> POINTER(POINTER(c_char)):  # Split text into multiple strings
	pass

# Append text at specific position and move cursor!
def text_append(text: bytes, append: bytes, position: POINTER(c_int)) -> None:
	pass

# Find first text occurrence within a string
def text_find_index(text: bytes, find: bytes) -> int:
	pass

# Get upper case version of provided string
def text_to_upper(text: bytes) -> bytes:
	pass

# Get lower case version of provided string
def text_to_lower(text: bytes) -> bytes:
	pass

# Get Pascal case notation version of provided string
def text_to_pascal(text: bytes) -> bytes:
	pass

# Get integer value from text (negative values not supported)
def text_to_integer(text: bytes) -> int:
	pass

# Draw a line in 3D world space
def draw_line_3d(startPos: Vector3, endPos: Vector3, color: Color) -> None:
	pass

# Draw a point in 3D space, actually a small line
def draw_point_3d(position: Vector3, color: Color) -> None:
	pass

# Draw a circle in 3D world space
def draw_circle_3d(center: Vector3, radius: float, rotationAxis: Vector3, rotationAngle: float, color: Color) -> None:
	pass

# Draw a color-filled triangle (vertex in counter-clockwise order!)
def draw_triangle_3d(v1: Vector3, v2: Vector3, v3: Vector3, color: Color) -> None:
	pass

# Draw a triangle strip defined by points
def draw_triangle_strip_3d(points: POINTER(Vector3), pointCount: int, color: Color) -> None:
	pass

# Draw cube
def draw_cube(position: Vector3, width: float, height: float, length: float, color: Color) -> None:
	pass

# Draw cube (Vector version)
def draw_cube_v(position: Vector3, size: Vector3, color: Color) -> None:
	pass

# Draw cube wires
def draw_cube_wires(position: Vector3, width: float, height: float, length: float, color: Color) -> None:
	pass

# Draw cube wires (Vector version)
def draw_cube_wires_v(position: Vector3, size: Vector3, color: Color) -> None:
	pass

# Draw cube textured
def draw_cube_texture(texture: Texture2D, position: Vector3, width: float, height: float, length: float, color: Color) -> None:
	pass

# Draw cube with a region of a texture
def draw_cube_texture_rec(texture: Texture2D, source: Rectangle, position: Vector3, width: float, height: float, length: float, color: Color) -> None:
	pass

# Draw sphere
def draw_sphere(centerPos: Vector3, radius: float, color: Color) -> None:
	pass

# Draw sphere with extended parameters
def draw_sphere_ex(centerPos: Vector3, radius: float, rings: int, slices: int, color: Color) -> None:
	pass

# Draw sphere wires
def draw_sphere_wires(centerPos: Vector3, radius: float, rings: int, slices: int, color: Color) -> None:
	pass

# Draw a cylinder/cone
def draw_cylinder(position: Vector3, radiusTop: float, radiusBottom: float, height: float, slices: int, color: Color) -> None:
	pass

# Draw a cylinder with base at startPos and top at endPos
def draw_cylinder_ex(startPos: Vector3, endPos: Vector3, startRadius: float, endRadius: float, sides: int, color: Color) -> None:
	pass

# Draw a cylinder/cone wires
def draw_cylinder_wires(position: Vector3, radiusTop: float, radiusBottom: float, height: float, slices: int, color: Color) -> None:
	pass

# Draw a cylinder wires with base at startPos and top at endPos
def draw_cylinder_wires_ex(startPos: Vector3, endPos: Vector3, startRadius: float, endRadius: float, sides: int, color: Color) -> None:
	pass

# Draw a plane XZ
def draw_plane(centerPos: Vector3, size: Vector2, color: Color) -> None:
	pass

# Draw a ray line
def draw_ray(ray: Ray, color: Color) -> None:
	pass

# Draw a grid (centered at (0, 0, 0))
def draw_grid(slices: int, spacing: float) -> None:
	pass

# Load model from files (meshes and materials)
def load_model(fileName: bytes) -> Model:
	pass

# Load model from generated mesh (default material)
def load_model_from_mesh(mesh: Mesh) -> Model:
	pass

# Unload model (including meshes) from memory (RAM and/or VRAM)
def unload_model(model: Model) -> None:
	pass

# Unload model (but not meshes) from memory (RAM and/or VRAM)
def unload_model_keep_meshes(model: Model) -> None:
	pass

# Compute model bounding box limits (considers all meshes)
def get_model_bounding_box(model: Model) -> BoundingBox:
	pass

# Draw a model (with texture if set)
def draw_model(model: Model, position: Vector3, scale: float, tint: Color) -> None:
	pass

# Draw a model with extended parameters
def draw_model_ex(model: Model, position: Vector3, rotationAxis: Vector3, rotationAngle: float, scale: Vector3, tint: Color) -> None:
	pass

# Draw a model wires (with texture if set)
def draw_model_wires(model: Model, position: Vector3, scale: float, tint: Color) -> None:
	pass

# Draw a model wires (with texture if set) with extended parameters
def draw_model_wires_ex(model: Model, position: Vector3, rotationAxis: Vector3, rotationAngle: float, scale: Vector3, tint: Color) -> None:
	pass

# Draw bounding box (wires)
def draw_bounding_box(box: BoundingBox, color: Color) -> None:
	pass

# Draw a billboard texture
def draw_billboard(camera: Camera, texture: Texture2D, position: Vector3, size: float, tint: Color) -> None:
	pass

# Draw a billboard texture defined by source
def draw_billboard_rec(camera: Camera, texture: Texture2D, source: Rectangle, position: Vector3, size: Vector2, tint: Color) -> None:
	pass

# Draw a billboard texture defined by source and rotation
def draw_billboard_pro(camera: Camera, texture: Texture2D, source: Rectangle, position: Vector3, up: Vector3, size: Vector2, origin: Vector2, rotation: float, tint: Color) -> None:
	pass

# Upload mesh vertex data in GPU and provide VAO/VBO ids
def upload_mesh(mesh: POINTER(Mesh), dynamic: bool) -> None:
	pass

# Update mesh vertex data in GPU for a specific buffer index
def update_mesh_buffer(mesh: Mesh, index: int, data: int, dataSize: int, offset: int) -> None:
	pass

# Unload mesh data from CPU and GPU
def unload_mesh(mesh: Mesh) -> None:
	pass

# Draw a 3d mesh with material and transform
def draw_mesh(mesh: Mesh, material: Material, transform: Matrix) -> None:
	pass

# Draw multiple mesh instances with material and different transforms
def draw_mesh_instanced(mesh: Mesh, material: Material, transforms: POINTER(Matrix), instances: int) -> None:
	pass

# Export mesh data to file, returns true on success
def export_mesh(mesh: Mesh, fileName: bytes) -> bool:
	pass

# Compute mesh bounding box limits
def get_mesh_bounding_box(mesh: Mesh) -> BoundingBox:
	pass

# Compute mesh tangents
def gen_mesh_tangents(mesh: POINTER(Mesh)) -> None:
	pass

# Generate polygonal mesh
def gen_mesh_poly(sides: int, radius: float) -> Mesh:
	pass

# Generate plane mesh (with subdivisions)
def gen_mesh_plane(width: float, length: float, resX: int, resZ: int) -> Mesh:
	pass

# Generate cuboid mesh
def gen_mesh_cube(width: float, height: float, length: float) -> Mesh:
	pass

# Generate sphere mesh (standard sphere)
def gen_mesh_sphere(radius: float, rings: int, slices: int) -> Mesh:
	pass

# Generate half-sphere mesh (no bottom cap)
def gen_mesh_hemi_sphere(radius: float, rings: int, slices: int) -> Mesh:
	pass

# Generate cylinder mesh
def gen_mesh_cylinder(radius: float, height: float, slices: int) -> Mesh:
	pass

# Generate cone/pyramid mesh
def gen_mesh_cone(radius: float, height: float, slices: int) -> Mesh:
	pass

# Generate torus mesh
def gen_mesh_torus(radius: float, size: float, radSeg: int, sides: int) -> Mesh:
	pass

# Generate trefoil knot mesh
def gen_mesh_knot(radius: float, size: float, radSeg: int, sides: int) -> Mesh:
	pass

# Generate heightmap mesh from image data
def gen_mesh_heightmap(heightmap: Image, size: Vector3) -> Mesh:
	pass

# Generate cubes-based map mesh from image data
def gen_mesh_cubicmap(cubicmap: Image, cubeSize: Vector3) -> Mesh:
	pass

# Load materials from model file
def load_materials(fileName: bytes, materialCount: POINTER(c_int)) -> POINTER(Material):  # Load materials from model file
	pass

# Load default material (Supports: DIFFUSE, SPECULAR, NORMAL maps)
def load_material_default() -> Material:
	pass

# Unload material from GPU memory (VRAM)
def unload_material(material: Material) -> None:
	pass

# Set texture for a material map type (MATERIAL_MAP_DIFFUSE, MATERIAL_MAP_SPECULAR...)
def set_material_texture(material: POINTER(Material), mapType: int, texture: Texture2D) -> None:
	pass

# Set material for a mesh
def set_model_mesh_material(model: POINTER(Model), meshId: int, materialId: int) -> None:
	pass

# Load model animations from file
def load_model_animations(fileName: bytes, animCount: POINTER(c_uint)) -> POINTER(ModelAnimation):  # Load model animations from file
	pass

# Update model animation pose
def update_model_animation(model: Model, anim: ModelAnimation, frame: int) -> None:
	pass

# Unload animation data
def unload_model_animation(anim: ModelAnimation) -> None:
	pass

# Unload animation array data
def unload_model_animations(animations: POINTER(ModelAnimation), count: int) -> None:
	pass

# Check model animation skeleton match
def is_model_animation_valid(model: Model, anim: ModelAnimation) -> bool:
	pass

# Check collision between two spheres
def check_collision_spheres(center1: Vector3, radius1: float, center2: Vector3, radius2: float) -> bool:
	pass

# Check collision between two bounding boxes
def check_collision_boxes(box1: BoundingBox, box2: BoundingBox) -> bool:
	pass

# Check collision between box and sphere
def check_collision_box_sphere(box: BoundingBox, center: Vector3, radius: float) -> bool:
	pass

# Get collision info between ray and sphere
def get_ray_collision_sphere(ray: Ray, center: Vector3, radius: float) -> RayCollision:
	pass

# Get collision info between ray and box
def get_ray_collision_box(ray: Ray, box: BoundingBox) -> RayCollision:
	pass

# Get collision info between ray and mesh
def get_ray_collision_mesh(ray: Ray, mesh: Mesh, transform: Matrix) -> RayCollision:
	pass

# Get collision info between ray and triangle
def get_ray_collision_triangle(ray: Ray, p1: Vector3, p2: Vector3, p3: Vector3) -> RayCollision:
	pass

# Get collision info between ray and quad
def get_ray_collision_quad(ray: Ray, p1: Vector3, p2: Vector3, p3: Vector3, p4: Vector3) -> RayCollision:
	pass

# Initialize audio device and context
def init_audio_device() -> None:
	pass

# Close the audio device and context
def close_audio_device() -> None:
	pass

# Check if audio device has been initialized successfully
def is_audio_device_ready() -> bool:
	pass

# Set master volume (listener)
def set_master_volume(volume: float) -> None:
	pass

# Stop any sound playing (using multichannel buffer pool)
def stop_sound_multi() -> None:
	pass

# Get number of sounds playing in the multichannel
def get_sounds_playing() -> int:
	pass

# Unload samples data loaded with LoadWaveSamples()
def unload_wave_samples(samples: POINTER(c_float)) -> None:
	pass

# Default size for new audio streams
def set_audio_stream_buffer_size_default(size: int) -> None:
	pass




from enum import IntEnum

#  System/Window config flags
class ConfigFlags(IntEnum):
	FLAG_VSYNC_HINT: int  # Set to try enabling V-Sync on GPU
	FLAG_FULLSCREEN_MODE: int  # Set to run program in fullscreen
	FLAG_WINDOW_RESIZABLE: int  # Set to allow resizable window
	FLAG_WINDOW_UNDECORATED: int  # Set to disable window decoration (frame and buttons)
	FLAG_WINDOW_HIDDEN: int  # Set to hide window
	FLAG_WINDOW_MINIMIZED: int  # Set to minimize window (iconify)
	FLAG_WINDOW_MAXIMIZED: int  # Set to maximize window (expanded to monitor)
	FLAG_WINDOW_UNFOCUSED: int  # Set to window non focused
	FLAG_WINDOW_TOPMOST: int  # Set to window always on top
	FLAG_WINDOW_ALWAYS_RUN: int  # Set to allow windows running while minimized
	FLAG_WINDOW_TRANSPARENT: int  # Set to allow transparent framebuffer
	FLAG_WINDOW_HIGHDPI: int  # Set to support HighDPI
	FLAG_WINDOW_MOUSE_PASSTHROUGH: int  # Set to support mouse passthrough, only supported when FLAG_WINDOW_UNDECORATED
	FLAG_MSAA_4X_HINT: int  # Set to try enabling MSAA 4X
	FLAG_INTERLACED_HINT: int  # Set to try enabling interlaced video format (for V3D)


#  Trace log level
class TraceLogLevel(IntEnum):
	LOG_ALL: int  # Display all logs
	LOG_TRACE: int  # Trace logging, intended for internal use only
	LOG_DEBUG: int  # Debug logging, used for internal debugging, it should be disabled on release builds
	LOG_INFO: int  # Info logging, used for program execution info
	LOG_WARNING: int  # Warning logging, used on recoverable failures
	LOG_ERROR: int  # Error logging, used on unrecoverable failures
	LOG_FATAL: int  # Fatal logging, used to abort program: exit(EXIT_FAILURE)
	LOG_NONE: int  # Disable logging


#  Keyboard keys (US keyboard layout)
class KeyboardKey(IntEnum):
	KEY_NULL: int  # Key: NULL, used for no key pressed
	KEY_APOSTROPHE: int  # Key: '
	KEY_COMMA: int  # Key: ,
	KEY_MINUS: int  # Key: -
	KEY_PERIOD: int  # Key: .
	KEY_SLASH: int  # Key: /
	KEY_ZERO: int  # Key: 0
	KEY_ONE: int  # Key: 1
	KEY_TWO: int  # Key: 2
	KEY_THREE: int  # Key: 3
	KEY_FOUR: int  # Key: 4
	KEY_FIVE: int  # Key: 5
	KEY_SIX: int  # Key: 6
	KEY_SEVEN: int  # Key: 7
	KEY_EIGHT: int  # Key: 8
	KEY_NINE: int  # Key: 9
	KEY_SEMICOLON: int  # Key: ;
	KEY_EQUAL: int  # Key: =
	KEY_A: int  # Key: A | a
	KEY_B: int  # Key: B | b
	KEY_C: int  # Key: C | c
	KEY_D: int  # Key: D | d
	KEY_E: int  # Key: E | e
	KEY_F: int  # Key: F | f
	KEY_G: int  # Key: G | g
	KEY_H: int  # Key: H | h
	KEY_I: int  # Key: I | i
	KEY_J: int  # Key: J | j
	KEY_K: int  # Key: K | k
	KEY_L: int  # Key: L | l
	KEY_M: int  # Key: M | m
	KEY_N: int  # Key: N | n
	KEY_O: int  # Key: O | o
	KEY_P: int  # Key: P | p
	KEY_Q: int  # Key: Q | q
	KEY_R: int  # Key: R | r
	KEY_S: int  # Key: S | s
	KEY_T: int  # Key: T | t
	KEY_U: int  # Key: U | u
	KEY_V: int  # Key: V | v
	KEY_W: int  # Key: W | w
	KEY_X: int  # Key: X | x
	KEY_Y: int  # Key: Y | y
	KEY_Z: int  # Key: Z | z
	KEY_LEFT_BRACKET: int  # Key: [
	KEY_BACKSLASH: int  # Key: '\'
	KEY_RIGHT_BRACKET: int  # Key: ]
	KEY_GRAVE: int  # Key: `
	KEY_SPACE: int  # Key: Space
	KEY_ESCAPE: int  # Key: Esc
	KEY_ENTER: int  # Key: Enter
	KEY_TAB: int  # Key: Tab
	KEY_BACKSPACE: int  # Key: Backspace
	KEY_INSERT: int  # Key: Ins
	KEY_DELETE: int  # Key: Del
	KEY_RIGHT: int  # Key: Cursor right
	KEY_LEFT: int  # Key: Cursor left
	KEY_DOWN: int  # Key: Cursor down
	KEY_UP: int  # Key: Cursor up
	KEY_PAGE_UP: int  # Key: Page up
	KEY_PAGE_DOWN: int  # Key: Page down
	KEY_HOME: int  # Key: Home
	KEY_END: int  # Key: End
	KEY_CAPS_LOCK: int  # Key: Caps lock
	KEY_SCROLL_LOCK: int  # Key: Scroll down
	KEY_NUM_LOCK: int  # Key: Num lock
	KEY_PRINT_SCREEN: int  # Key: Print screen
	KEY_PAUSE: int  # Key: Pause
	KEY_F1: int  # Key: F1
	KEY_F2: int  # Key: F2
	KEY_F3: int  # Key: F3
	KEY_F4: int  # Key: F4
	KEY_F5: int  # Key: F5
	KEY_F6: int  # Key: F6
	KEY_F7: int  # Key: F7
	KEY_F8: int  # Key: F8
	KEY_F9: int  # Key: F9
	KEY_F10: int  # Key: F10
	KEY_F11: int  # Key: F11
	KEY_F12: int  # Key: F12
	KEY_LEFT_SHIFT: int  # Key: Shift left
	KEY_LEFT_CONTROL: int  # Key: Control left
	KEY_LEFT_ALT: int  # Key: Alt left
	KEY_LEFT_SUPER: int  # Key: Super left
	KEY_RIGHT_SHIFT: int  # Key: Shift right
	KEY_RIGHT_CONTROL: int  # Key: Control right
	KEY_RIGHT_ALT: int  # Key: Alt right
	KEY_RIGHT_SUPER: int  # Key: Super right
	KEY_KB_MENU: int  # Key: KB menu
	KEY_KP_0: int  # Key: Keypad 0
	KEY_KP_1: int  # Key: Keypad 1
	KEY_KP_2: int  # Key: Keypad 2
	KEY_KP_3: int  # Key: Keypad 3
	KEY_KP_4: int  # Key: Keypad 4
	KEY_KP_5: int  # Key: Keypad 5
	KEY_KP_6: int  # Key: Keypad 6
	KEY_KP_7: int  # Key: Keypad 7
	KEY_KP_8: int  # Key: Keypad 8
	KEY_KP_9: int  # Key: Keypad 9
	KEY_KP_DECIMAL: int  # Key: Keypad .
	KEY_KP_DIVIDE: int  # Key: Keypad /
	KEY_KP_MULTIPLY: int  # Key: Keypad *
	KEY_KP_SUBTRACT: int  # Key: Keypad -
	KEY_KP_ADD: int  # Key: Keypad +
	KEY_KP_ENTER: int  # Key: Keypad Enter
	KEY_KP_EQUAL: int  # Key: Keypad =
	KEY_BACK: int  # Key: Android back button
	KEY_MENU: int  # Key: Android menu button
	KEY_VOLUME_UP: int  # Key: Android volume up button
	KEY_VOLUME_DOWN: int  # Key: Android volume down button


#  Mouse buttons
class MouseButton(IntEnum):
	MOUSE_BUTTON_LEFT: int  # Mouse button left
	MOUSE_BUTTON_RIGHT: int  # Mouse button right
	MOUSE_BUTTON_MIDDLE: int  # Mouse button middle (pressed wheel)
	MOUSE_BUTTON_SIDE: int  # Mouse button side (advanced mouse device)
	MOUSE_BUTTON_EXTRA: int  # Mouse button extra (advanced mouse device)
	MOUSE_BUTTON_FORWARD: int  # Mouse button fordward (advanced mouse device)
	MOUSE_BUTTON_BACK: int  # Mouse button back (advanced mouse device)


#  Mouse cursor
class MouseCursor(IntEnum):
	MOUSE_CURSOR_DEFAULT: int  # Default pointer shape
	MOUSE_CURSOR_ARROW: int  # Arrow shape
	MOUSE_CURSOR_IBEAM: int  # Text writing cursor shape
	MOUSE_CURSOR_CROSSHAIR: int  # Cross shape
	MOUSE_CURSOR_POINTING_HAND: int  # Pointing hand cursor
	MOUSE_CURSOR_RESIZE_EW: int  # Horizontal resize/move arrow shape
	MOUSE_CURSOR_RESIZE_NS: int  # Vertical resize/move arrow shape
	MOUSE_CURSOR_RESIZE_NWSE: int  # Top-left to bottom-right diagonal resize/move arrow shape
	MOUSE_CURSOR_RESIZE_NESW: int  # The top-right to bottom-left diagonal resize/move arrow shape
	MOUSE_CURSOR_RESIZE_ALL: int  # The omni-directional resize/move cursor shape
	MOUSE_CURSOR_NOT_ALLOWED: int  # The operation-not-allowed shape


#  Gamepad buttons
class GamepadButton(IntEnum):
	GAMEPAD_BUTTON_UNKNOWN: int  # Unknown button, just for error checking
	GAMEPAD_BUTTON_LEFT_FACE_UP: int  # Gamepad left DPAD up button
	GAMEPAD_BUTTON_LEFT_FACE_RIGHT: int  # Gamepad left DPAD right button
	GAMEPAD_BUTTON_LEFT_FACE_DOWN: int  # Gamepad left DPAD down button
	GAMEPAD_BUTTON_LEFT_FACE_LEFT: int  # Gamepad left DPAD left button
	GAMEPAD_BUTTON_RIGHT_FACE_UP: int  # Gamepad right button up (i.e. PS3: Triangle, Xbox: Y)
	GAMEPAD_BUTTON_RIGHT_FACE_RIGHT: int  # Gamepad right button right (i.e. PS3: Square, Xbox: X)
	GAMEPAD_BUTTON_RIGHT_FACE_DOWN: int  # Gamepad right button down (i.e. PS3: Cross, Xbox: A)
	GAMEPAD_BUTTON_RIGHT_FACE_LEFT: int  # Gamepad right button left (i.e. PS3: Circle, Xbox: B)
	GAMEPAD_BUTTON_LEFT_TRIGGER_1: int  # Gamepad top/back trigger left (first), it could be a trailing button
	GAMEPAD_BUTTON_LEFT_TRIGGER_2: int  # Gamepad top/back trigger left (second), it could be a trailing button
	GAMEPAD_BUTTON_RIGHT_TRIGGER_1: int  # Gamepad top/back trigger right (one), it could be a trailing button
	GAMEPAD_BUTTON_RIGHT_TRIGGER_2: int  # Gamepad top/back trigger right (second), it could be a trailing button
	GAMEPAD_BUTTON_MIDDLE_LEFT: int  # Gamepad center buttons, left one (i.e. PS3: Select)
	GAMEPAD_BUTTON_MIDDLE: int  # Gamepad center buttons, middle one (i.e. PS3: PS, Xbox: XBOX)
	GAMEPAD_BUTTON_MIDDLE_RIGHT: int  # Gamepad center buttons, right one (i.e. PS3: Start)
	GAMEPAD_BUTTON_LEFT_THUMB: int  # Gamepad joystick pressed button left
	GAMEPAD_BUTTON_RIGHT_THUMB: int  # Gamepad joystick pressed button right


#  Gamepad axis
class GamepadAxis(IntEnum):
	GAMEPAD_AXIS_LEFT_X: int  # Gamepad left stick X axis
	GAMEPAD_AXIS_LEFT_Y: int  # Gamepad left stick Y axis
	GAMEPAD_AXIS_RIGHT_X: int  # Gamepad right stick X axis
	GAMEPAD_AXIS_RIGHT_Y: int  # Gamepad right stick Y axis
	GAMEPAD_AXIS_LEFT_TRIGGER: int  # Gamepad back trigger left, pressure level: [1..-1]
	GAMEPAD_AXIS_RIGHT_TRIGGER: int  # Gamepad back trigger right, pressure level: [1..-1]


#  Material map index
class MaterialMapIndex(IntEnum):
	MATERIAL_MAP_ALBEDO: int  # Albedo material (same as: MATERIAL_MAP_DIFFUSE)
	MATERIAL_MAP_METALNESS: int  # Metalness material (same as: MATERIAL_MAP_SPECULAR)
	MATERIAL_MAP_NORMAL: int  # Normal material
	MATERIAL_MAP_ROUGHNESS: int  # Roughness material
	MATERIAL_MAP_OCCLUSION: int  # Ambient occlusion material
	MATERIAL_MAP_EMISSION: int  # Emission material
	MATERIAL_MAP_HEIGHT: int  # Heightmap material
	MATERIAL_MAP_CUBEMAP: int  # Cubemap material (NOTE: Uses GL_TEXTURE_CUBE_MAP)
	MATERIAL_MAP_IRRADIANCE: int  # Irradiance material (NOTE: Uses GL_TEXTURE_CUBE_MAP)
	MATERIAL_MAP_PREFILTER: int  # Prefilter material (NOTE: Uses GL_TEXTURE_CUBE_MAP)
	MATERIAL_MAP_BRDF: int  # Brdf material


#  Shader location index
class ShaderLocationIndex(IntEnum):
	SHADER_LOC_VERTEX_POSITION: int  # Shader location: vertex attribute: position
	SHADER_LOC_VERTEX_TEXCOORD01: int  # Shader location: vertex attribute: texcoord01
	SHADER_LOC_VERTEX_TEXCOORD02: int  # Shader location: vertex attribute: texcoord02
	SHADER_LOC_VERTEX_NORMAL: int  # Shader location: vertex attribute: normal
	SHADER_LOC_VERTEX_TANGENT: int  # Shader location: vertex attribute: tangent
	SHADER_LOC_VERTEX_COLOR: int  # Shader location: vertex attribute: color
	SHADER_LOC_MATRIX_MVP: int  # Shader location: matrix uniform: model-view-projection
	SHADER_LOC_MATRIX_VIEW: int  # Shader location: matrix uniform: view (camera transform)
	SHADER_LOC_MATRIX_PROJECTION: int  # Shader location: matrix uniform: projection
	SHADER_LOC_MATRIX_MODEL: int  # Shader location: matrix uniform: model (transform)
	SHADER_LOC_MATRIX_NORMAL: int  # Shader location: matrix uniform: normal
	SHADER_LOC_VECTOR_VIEW: int  # Shader location: vector uniform: view
	SHADER_LOC_COLOR_DIFFUSE: int  # Shader location: vector uniform: diffuse color
	SHADER_LOC_COLOR_SPECULAR: int  # Shader location: vector uniform: specular color
	SHADER_LOC_COLOR_AMBIENT: int  # Shader location: vector uniform: ambient color
	SHADER_LOC_MAP_ALBEDO: int  # Shader location: sampler2d texture: albedo (same as: SHADER_LOC_MAP_DIFFUSE)
	SHADER_LOC_MAP_METALNESS: int  # Shader location: sampler2d texture: metalness (same as: SHADER_LOC_MAP_SPECULAR)
	SHADER_LOC_MAP_NORMAL: int  # Shader location: sampler2d texture: normal
	SHADER_LOC_MAP_ROUGHNESS: int  # Shader location: sampler2d texture: roughness
	SHADER_LOC_MAP_OCCLUSION: int  # Shader location: sampler2d texture: occlusion
	SHADER_LOC_MAP_EMISSION: int  # Shader location: sampler2d texture: emission
	SHADER_LOC_MAP_HEIGHT: int  # Shader location: sampler2d texture: height
	SHADER_LOC_MAP_CUBEMAP: int  # Shader location: samplerCube texture: cubemap
	SHADER_LOC_MAP_IRRADIANCE: int  # Shader location: samplerCube texture: irradiance
	SHADER_LOC_MAP_PREFILTER: int  # Shader location: samplerCube texture: prefilter
	SHADER_LOC_MAP_BRDF: int  # Shader location: sampler2d texture: brdf


#  Shader uniform data type
class ShaderUniformDataType(IntEnum):
	SHADER_UNIFORM_FLOAT: int  # Shader uniform type: float
	SHADER_UNIFORM_VEC2: int  # Shader uniform type: vec2 (2 float)
	SHADER_UNIFORM_VEC3: int  # Shader uniform type: vec3 (3 float)
	SHADER_UNIFORM_VEC4: int  # Shader uniform type: vec4 (4 float)
	SHADER_UNIFORM_INT: int  # Shader uniform type: int
	SHADER_UNIFORM_IVEC2: int  # Shader uniform type: ivec2 (2 int)
	SHADER_UNIFORM_IVEC3: int  # Shader uniform type: ivec3 (3 int)
	SHADER_UNIFORM_IVEC4: int  # Shader uniform type: ivec4 (4 int)
	SHADER_UNIFORM_SAMPLER2D: int  # Shader uniform type: sampler2d


#  Shader attribute data types
class ShaderAttributeDataType(IntEnum):
	SHADER_ATTRIB_FLOAT: int  # Shader attribute type: float
	SHADER_ATTRIB_VEC2: int  # Shader attribute type: vec2 (2 float)
	SHADER_ATTRIB_VEC3: int  # Shader attribute type: vec3 (3 float)
	SHADER_ATTRIB_VEC4: int  # Shader attribute type: vec4 (4 float)


#  Pixel formats
class PixelFormat(IntEnum):
	PIXELFORMAT_UNCOMPRESSED_GRAYSCALE: int  # 8 bit per pixel (no alpha)
	PIXELFORMAT_UNCOMPRESSED_GRAY_ALPHA: int  # 8*2 bpp (2 channels)
	PIXELFORMAT_UNCOMPRESSED_R5G6B5: int  # 16 bpp
	PIXELFORMAT_UNCOMPRESSED_R8G8B8: int  # 24 bpp
	PIXELFORMAT_UNCOMPRESSED_R5G5B5A1: int  # 16 bpp (1 bit alpha)
	PIXELFORMAT_UNCOMPRESSED_R4G4B4A4: int  # 16 bpp (4 bit alpha)
	PIXELFORMAT_UNCOMPRESSED_R8G8B8A8: int  # 32 bpp
	PIXELFORMAT_UNCOMPRESSED_R32: int  # 32 bpp (1 channel - float)
	PIXELFORMAT_UNCOMPRESSED_R32G32B32: int  # 32*3 bpp (3 channels - float)
	PIXELFORMAT_UNCOMPRESSED_R32G32B32A32: int  # 32*4 bpp (4 channels - float)
	PIXELFORMAT_COMPRESSED_DXT1_RGB: int  # 4 bpp (no alpha)
	PIXELFORMAT_COMPRESSED_DXT1_RGBA: int  # 4 bpp (1 bit alpha)
	PIXELFORMAT_COMPRESSED_DXT3_RGBA: int  # 8 bpp
	PIXELFORMAT_COMPRESSED_DXT5_RGBA: int  # 8 bpp
	PIXELFORMAT_COMPRESSED_ETC1_RGB: int  # 4 bpp
	PIXELFORMAT_COMPRESSED_ETC2_RGB: int  # 4 bpp
	PIXELFORMAT_COMPRESSED_ETC2_EAC_RGBA: int  # 8 bpp
	PIXELFORMAT_COMPRESSED_PVRT_RGB: int  # 4 bpp
	PIXELFORMAT_COMPRESSED_PVRT_RGBA: int  # 4 bpp
	PIXELFORMAT_COMPRESSED_ASTC_4x4_RGBA: int  # 8 bpp
	PIXELFORMAT_COMPRESSED_ASTC_8x8_RGBA: int  # 2 bpp


#  Texture parameters: filter mode
class TextureFilter(IntEnum):
	TEXTURE_FILTER_POINT: int  # No filter, just pixel approximation
	TEXTURE_FILTER_BILINEAR: int  # Linear filtering
	TEXTURE_FILTER_TRILINEAR: int  # Trilinear filtering (linear with mipmaps)
	TEXTURE_FILTER_ANISOTROPIC_4X: int  # Anisotropic filtering 4x
	TEXTURE_FILTER_ANISOTROPIC_8X: int  # Anisotropic filtering 8x
	TEXTURE_FILTER_ANISOTROPIC_16X: int  # Anisotropic filtering 16x


#  Texture parameters: wrap mode
class TextureWrap(IntEnum):
	TEXTURE_WRAP_REPEAT: int  # Repeats texture in tiled mode
	TEXTURE_WRAP_CLAMP: int  # Clamps texture to edge pixel in tiled mode
	TEXTURE_WRAP_MIRROR_REPEAT: int  # Mirrors and repeats the texture in tiled mode
	TEXTURE_WRAP_MIRROR_CLAMP: int  # Mirrors and clamps to border the texture in tiled mode


#  Cubemap layouts
class CubemapLayout(IntEnum):
	CUBEMAP_LAYOUT_AUTO_DETECT: int  # Automatically detect layout type
	CUBEMAP_LAYOUT_LINE_VERTICAL: int  # Layout is defined by a vertical line with faces
	CUBEMAP_LAYOUT_LINE_HORIZONTAL: int  # Layout is defined by an horizontal line with faces
	CUBEMAP_LAYOUT_CROSS_THREE_BY_FOUR: int  # Layout is defined by a 3x4 cross with cubemap faces
	CUBEMAP_LAYOUT_CROSS_FOUR_BY_THREE: int  # Layout is defined by a 4x3 cross with cubemap faces
	CUBEMAP_LAYOUT_PANORAMA: int  # Layout is defined by a panorama image (equirectangular map)


#  Font type, defines generation method
class FontType(IntEnum):
	FONT_DEFAULT: int  # Default font generation, anti-aliased
	FONT_BITMAP: int  # Bitmap font generation, no anti-aliasing
	FONT_SDF: int  # SDF font generation, requires external shader


#  Color blending modes (pre-defined)
class BlendMode(IntEnum):
	BLEND_ALPHA: int  # Blend textures considering alpha (default)
	BLEND_ADDITIVE: int  # Blend textures adding colors
	BLEND_MULTIPLIED: int  # Blend textures multiplying colors
	BLEND_ADD_COLORS: int  # Blend textures adding colors (alternative)
	BLEND_SUBTRACT_COLORS: int  # Blend textures subtracting colors (alternative)
	BLEND_ALPHA_PREMULTIPLY: int  # Blend premultiplied textures considering alpha
	BLEND_CUSTOM: int  # Blend textures using custom src/dst factors (use rlSetBlendMode())


#  Gesture
class Gesture(IntEnum):
	GESTURE_NONE: int  # No gesture
	GESTURE_TAP: int  # Tap gesture
	GESTURE_DOUBLETAP: int  # Double tap gesture
	GESTURE_HOLD: int  # Hold gesture
	GESTURE_DRAG: int  # Drag gesture
	GESTURE_SWIPE_RIGHT: int  # Swipe right gesture
	GESTURE_SWIPE_LEFT: int  # Swipe left gesture
	GESTURE_SWIPE_UP: int  # Swipe up gesture
	GESTURE_SWIPE_DOWN: int  # Swipe down gesture
	GESTURE_PINCH_IN: int  # Pinch in gesture
	GESTURE_PINCH_OUT: int  # Pinch out gesture


#  Camera system modes
class CameraMode(IntEnum):
	CAMERA_CUSTOM: int  # Custom camera
	CAMERA_FREE: int  # Free camera
	CAMERA_ORBITAL: int  # Orbital camera
	CAMERA_FIRST_PERSON: int  # First person camera
	CAMERA_THIRD_PERSON: int  # Third person camera


#  Camera projection
class CameraProjection(IntEnum):
	CAMERA_PERSPECTIVE: int  # Perspective projection
	CAMERA_ORTHOGRAPHIC: int  # Orthographic projection


#  N-patch layout
class NPatchLayout(IntEnum):
	NPATCH_NINE_PATCH: int  # Npatch layout: 3x3 tiles
	NPATCH_THREE_PATCH_VERTICAL: int  # Npatch layout: 1x3 tiles
	NPATCH_THREE_PATCH_HORIZONTAL: int  # Npatch layout: 3x1 tiles





from ctypes import *


#  Vector2, 2 components
class Vector2(Structure):
	@property
	def x(self) -> float:
		pass

	@x.setter
	def x(self, i: float) -> None:
		pass

	@property
	def y(self) -> float:
		pass

	@y.setter
	def y(self, i: float) -> None:
		pass


#  Vector3, 3 components
class Vector3(Structure):
	@property
	def x(self) -> float:
		pass

	@x.setter
	def x(self, i: float) -> None:
		pass

	@property
	def y(self) -> float:
		pass

	@y.setter
	def y(self, i: float) -> None:
		pass

	@property
	def z(self) -> float:
		pass

	@z.setter
	def z(self, i: float) -> None:
		pass


#  Vector4, 4 components
class Vector4(Structure):
	@property
	def x(self) -> float:
		pass

	@x.setter
	def x(self, i: float) -> None:
		pass

	@property
	def y(self) -> float:
		pass

	@y.setter
	def y(self, i: float) -> None:
		pass

	@property
	def z(self) -> float:
		pass

	@z.setter
	def z(self, i: float) -> None:
		pass

	@property
	def w(self) -> float:
		pass

	@w.setter
	def w(self, i: float) -> None:
		pass


#  Quaternion, 4 components (Vector4 alias)
class Quaternion(Structure):
	@property
	def x(self) -> float:
		pass

	@x.setter
	def x(self, i: float) -> None:
		pass

	@property
	def y(self) -> float:
		pass

	@y.setter
	def y(self, i: float) -> None:
		pass

	@property
	def z(self) -> float:
		pass

	@z.setter
	def z(self, i: float) -> None:
		pass

	@property
	def w(self) -> float:
		pass

	@w.setter
	def w(self, i: float) -> None:
		pass


#  Matrix, 4x4 components, column major, OpenGL style, right handed
class Matrix(Structure):
	@property
	def m0(self) -> float:
		pass

	@m0.setter
	def m0(self, i: float) -> None:
		pass

	@property
	def m4(self) -> float:
		pass

	@m4.setter
	def m4(self, i: float) -> None:
		pass

	@property
	def m8(self) -> float:
		pass

	@m8.setter
	def m8(self, i: float) -> None:
		pass

	@property
	def m12(self) -> float:
		pass

	@m12.setter
	def m12(self, i: float) -> None:
		pass

	@property
	def m1(self) -> float:
		pass

	@m1.setter
	def m1(self, i: float) -> None:
		pass

	@property
	def m5(self) -> float:
		pass

	@m5.setter
	def m5(self, i: float) -> None:
		pass

	@property
	def m9(self) -> float:
		pass

	@m9.setter
	def m9(self, i: float) -> None:
		pass

	@property
	def m13(self) -> float:
		pass

	@m13.setter
	def m13(self, i: float) -> None:
		pass

	@property
	def m2(self) -> float:
		pass

	@m2.setter
	def m2(self, i: float) -> None:
		pass

	@property
	def m6(self) -> float:
		pass

	@m6.setter
	def m6(self, i: float) -> None:
		pass

	@property
	def m10(self) -> float:
		pass

	@m10.setter
	def m10(self, i: float) -> None:
		pass

	@property
	def m14(self) -> float:
		pass

	@m14.setter
	def m14(self, i: float) -> None:
		pass

	@property
	def m3(self) -> float:
		pass

	@m3.setter
	def m3(self, i: float) -> None:
		pass

	@property
	def m7(self) -> float:
		pass

	@m7.setter
	def m7(self, i: float) -> None:
		pass

	@property
	def m11(self) -> float:
		pass

	@m11.setter
	def m11(self, i: float) -> None:
		pass

	@property
	def m15(self) -> float:
		pass

	@m15.setter
	def m15(self, i: float) -> None:
		pass


#  Color, 4 components, R8G8B8A8 (32bit)
class Color(Structure):
	@property
	def r(self) -> int:
		pass

	@r.setter
	def r(self, i: int) -> None:
		pass

	@property
	def g(self) -> int:
		pass

	@g.setter
	def g(self, i: int) -> None:
		pass

	@property
	def b(self) -> int:
		pass

	@b.setter
	def b(self, i: int) -> None:
		pass

	@property
	def a(self) -> int:
		pass

	@a.setter
	def a(self, i: int) -> None:
		pass


#  Rectangle, 4 components
class Rectangle(Structure):
	@property
	def x(self) -> float:
		pass

	@x.setter
	def x(self, i: float) -> None:
		pass

	@property
	def y(self) -> float:
		pass

	@y.setter
	def y(self, i: float) -> None:
		pass

	@property
	def width(self) -> float:
		pass

	@width.setter
	def width(self, i: float) -> None:
		pass

	@property
	def height(self) -> float:
		pass

	@height.setter
	def height(self, i: float) -> None:
		pass


#  Image, pixel data stored in CPU memory (RAM)
class Image(Structure):
	@property
	def data(self) -> int:
		pass

	@data.setter
	def data(self, i: int) -> None:
		pass

	@property
	def width(self) -> int:
		pass

	@width.setter
	def width(self, i: int) -> None:
		pass

	@property
	def height(self) -> int:
		pass

	@height.setter
	def height(self, i: int) -> None:
		pass

	@property
	def mipmaps(self) -> int:
		pass

	@mipmaps.setter
	def mipmaps(self, i: int) -> None:
		pass

	@property
	def format(self) -> int:
		pass

	@format.setter
	def format(self, i: int) -> None:
		pass


#  Texture, tex data stored in GPU memory (VRAM)
class Texture(Structure):
	@property
	def id(self) -> int:
		pass

	@id.setter
	def id(self, i: int) -> None:
		pass

	@property
	def width(self) -> int:
		pass

	@width.setter
	def width(self, i: int) -> None:
		pass

	@property
	def height(self) -> int:
		pass

	@height.setter
	def height(self, i: int) -> None:
		pass

	@property
	def mipmaps(self) -> int:
		pass

	@mipmaps.setter
	def mipmaps(self, i: int) -> None:
		pass

	@property
	def format(self) -> int:
		pass

	@format.setter
	def format(self, i: int) -> None:
		pass


#  Texture2D, same as Texture
class Texture2D(Structure):
	@property
	def id(self) -> int:
		pass

	@id.setter
	def id(self, i: int) -> None:
		pass

	@property
	def width(self) -> int:
		pass

	@width.setter
	def width(self, i: int) -> None:
		pass

	@property
	def height(self) -> int:
		pass

	@height.setter
	def height(self, i: int) -> None:
		pass

	@property
	def mipmaps(self) -> int:
		pass

	@mipmaps.setter
	def mipmaps(self, i: int) -> None:
		pass

	@property
	def format(self) -> int:
		pass

	@format.setter
	def format(self, i: int) -> None:
		pass


#  TextureCubemap, same as Texture
class TextureCubemap(Structure):
	@property
	def id(self) -> int:
		pass

	@id.setter
	def id(self, i: int) -> None:
		pass

	@property
	def width(self) -> int:
		pass

	@width.setter
	def width(self, i: int) -> None:
		pass

	@property
	def height(self) -> int:
		pass

	@height.setter
	def height(self, i: int) -> None:
		pass

	@property
	def mipmaps(self) -> int:
		pass

	@mipmaps.setter
	def mipmaps(self, i: int) -> None:
		pass

	@property
	def format(self) -> int:
		pass

	@format.setter
	def format(self, i: int) -> None:
		pass


#  RenderTexture, fbo for texture rendering
class RenderTexture(Structure):
	@property
	def id(self) -> int:
		pass

	@id.setter
	def id(self, i: int) -> None:
		pass

	@property
	def texture(self) -> Texture:
		pass

	@texture.setter
	def texture(self, i: Texture) -> None:
		pass

	@property
	def depth(self) -> Texture:
		pass

	@depth.setter
	def depth(self, i: Texture) -> None:
		pass


#  RenderTexture2D, same as RenderTexture
class RenderTexture2D(Structure):
	@property
	def id(self) -> int:
		pass

	@id.setter
	def id(self, i: int) -> None:
		pass

	@property
	def texture(self) -> Texture:
		pass

	@texture.setter
	def texture(self, i: Texture) -> None:
		pass

	@property
	def depth(self) -> Texture:
		pass

	@depth.setter
	def depth(self, i: Texture) -> None:
		pass


#  NPatchInfo, n-patch layout info
class NPatchInfo(Structure):
	@property
	def source(self) -> Rectangle:
		pass

	@source.setter
	def source(self, i: Rectangle) -> None:
		pass

	@property
	def left(self) -> int:
		pass

	@left.setter
	def left(self, i: int) -> None:
		pass

	@property
	def top(self) -> int:
		pass

	@top.setter
	def top(self, i: int) -> None:
		pass

	@property
	def right(self) -> int:
		pass

	@right.setter
	def right(self, i: int) -> None:
		pass

	@property
	def bottom(self) -> int:
		pass

	@bottom.setter
	def bottom(self, i: int) -> None:
		pass

	@property
	def layout(self) -> int:
		pass

	@layout.setter
	def layout(self, i: int) -> None:
		pass


#  GlyphInfo, font characters glyphs info
class GlyphInfo(Structure):
	@property
	def value(self) -> int:
		pass

	@value.setter
	def value(self, i: int) -> None:
		pass

	@property
	def offsetX(self) -> int:
		pass

	@offsetX.setter
	def offsetX(self, i: int) -> None:
		pass

	@property
	def offsetY(self) -> int:
		pass

	@offsetY.setter
	def offsetY(self, i: int) -> None:
		pass

	@property
	def advanceX(self) -> int:
		pass

	@advanceX.setter
	def advanceX(self, i: int) -> None:
		pass

	@property
	def image(self) -> Image:
		pass

	@image.setter
	def image(self, i: Image) -> None:
		pass


#  Font, font texture and GlyphInfo array data
class Font(Structure):
	@property
	def baseSize(self) -> int:
		pass

	@baseSize.setter
	def baseSize(self, i: int) -> None:
		pass

	@property
	def glyphCount(self) -> int:
		pass

	@glyphCount.setter
	def glyphCount(self, i: int) -> None:
		pass

	@property
	def glyphPadding(self) -> int:
		pass

	@glyphPadding.setter
	def glyphPadding(self, i: int) -> None:
		pass

	@property
	def texture(self) -> Texture2D:
		pass

	@texture.setter
	def texture(self, i: Texture2D) -> None:
		pass

	@property
	def recs(self) -> POINTER(Rectangle):
		pass

	@recs.setter
	def recs(self, i: POINTER(Rectangle)) -> None:
		pass

	@property
	def glyphs(self) -> POINTER(GlyphInfo):
		pass

	@glyphs.setter
	def glyphs(self, i: POINTER(GlyphInfo)) -> None:
		pass


#  Camera, defines position/orientation in 3d space
class Camera3D(Structure):
	@property
	def position(self) -> Vector3:
		pass

	@position.setter
	def position(self, i: Vector3) -> None:
		pass

	@property
	def target(self) -> Vector3:
		pass

	@target.setter
	def target(self, i: Vector3) -> None:
		pass

	@property
	def up(self) -> Vector3:
		pass

	@up.setter
	def up(self, i: Vector3) -> None:
		pass

	@property
	def fovy(self) -> float:
		pass

	@fovy.setter
	def fovy(self, i: float) -> None:
		pass

	@property
	def projection(self) -> int:
		pass

	@projection.setter
	def projection(self, i: int) -> None:
		pass


#  Camera type fallback, defaults to Camera3D
class Camera(Structure):
	@property
	def position(self) -> Vector3:
		pass

	@position.setter
	def position(self, i: Vector3) -> None:
		pass

	@property
	def target(self) -> Vector3:
		pass

	@target.setter
	def target(self, i: Vector3) -> None:
		pass

	@property
	def up(self) -> Vector3:
		pass

	@up.setter
	def up(self, i: Vector3) -> None:
		pass

	@property
	def fovy(self) -> float:
		pass

	@fovy.setter
	def fovy(self, i: float) -> None:
		pass

	@property
	def projection(self) -> int:
		pass

	@projection.setter
	def projection(self, i: int) -> None:
		pass


#  Camera2D, defines position/orientation in 2d space
class Camera2D(Structure):
	@property
	def offset(self) -> Vector2:
		pass

	@offset.setter
	def offset(self, i: Vector2) -> None:
		pass

	@property
	def target(self) -> Vector2:
		pass

	@target.setter
	def target(self, i: Vector2) -> None:
		pass

	@property
	def rotation(self) -> float:
		pass

	@rotation.setter
	def rotation(self, i: float) -> None:
		pass

	@property
	def zoom(self) -> float:
		pass

	@zoom.setter
	def zoom(self, i: float) -> None:
		pass


#  Mesh, vertex data and vao/vbo
class Mesh(Structure):
	@property
	def vertexCount(self) -> int:
		pass

	@vertexCount.setter
	def vertexCount(self, i: int) -> None:
		pass

	@property
	def triangleCount(self) -> int:
		pass

	@triangleCount.setter
	def triangleCount(self, i: int) -> None:
		pass

	@property
	def vertices(self) -> POINTER(c_float):
		pass

	@vertices.setter
	def vertices(self, i: POINTER(c_float)) -> None:
		pass

	@property
	def texcoords(self) -> POINTER(c_float):
		pass

	@texcoords.setter
	def texcoords(self, i: POINTER(c_float)) -> None:
		pass

	@property
	def texcoords2(self) -> POINTER(c_float):
		pass

	@texcoords2.setter
	def texcoords2(self, i: POINTER(c_float)) -> None:
		pass

	@property
	def normals(self) -> POINTER(c_float):
		pass

	@normals.setter
	def normals(self, i: POINTER(c_float)) -> None:
		pass

	@property
	def tangents(self) -> POINTER(c_float):
		pass

	@tangents.setter
	def tangents(self, i: POINTER(c_float)) -> None:
		pass

	@property
	def colors(self) -> POINTER(c_ubyte):
		pass

	@colors.setter
	def colors(self, i: POINTER(c_ubyte)) -> None:
		pass

	@property
	def indices(self) -> POINTER(c_ushort):
		pass

	@indices.setter
	def indices(self, i: POINTER(c_ushort)) -> None:
		pass

	@property
	def animVertices(self) -> POINTER(c_float):
		pass

	@animVertices.setter
	def animVertices(self, i: POINTER(c_float)) -> None:
		pass

	@property
	def animNormals(self) -> POINTER(c_float):
		pass

	@animNormals.setter
	def animNormals(self, i: POINTER(c_float)) -> None:
		pass

	@property
	def boneIds(self) -> POINTER(c_ubyte):
		pass

	@boneIds.setter
	def boneIds(self, i: POINTER(c_ubyte)) -> None:
		pass

	@property
	def boneWeights(self) -> POINTER(c_float):
		pass

	@boneWeights.setter
	def boneWeights(self, i: POINTER(c_float)) -> None:
		pass

	@property
	def vaoId(self) -> int:
		pass

	@vaoId.setter
	def vaoId(self, i: int) -> None:
		pass

	@property
	def vboId(self) -> POINTER(c_uint):
		pass

	@vboId.setter
	def vboId(self, i: POINTER(c_uint)) -> None:
		pass


#  Shader
class Shader(Structure):
	@property
	def id(self) -> int:
		pass

	@id.setter
	def id(self, i: int) -> None:
		pass

	@property
	def locs(self) -> POINTER(c_int):
		pass

	@locs.setter
	def locs(self, i: POINTER(c_int)) -> None:
		pass


#  MaterialMap
class MaterialMap(Structure):
	@property
	def texture(self) -> Texture2D:
		pass

	@texture.setter
	def texture(self, i: Texture2D) -> None:
		pass

	@property
	def color(self) -> Color:
		pass

	@color.setter
	def color(self, i: Color) -> None:
		pass

	@property
	def value(self) -> float:
		pass

	@value.setter
	def value(self, i: float) -> None:
		pass


#  Material, includes shader and maps
class Material(Structure):
	@property
	def shader(self) -> Shader:
		pass

	@shader.setter
	def shader(self, i: Shader) -> None:
		pass

	@property
	def maps(self) -> POINTER(MaterialMap):
		pass

	@maps.setter
	def maps(self, i: POINTER(MaterialMap)) -> None:
		pass

	@property
	def params(self) -> c_float * 4:
		pass

	@params.setter
	def params(self, i: c_float * 4) -> None:
		pass


#  Transform, vectex transformation data
class Transform(Structure):
	@property
	def translation(self) -> Vector3:
		pass

	@translation.setter
	def translation(self, i: Vector3) -> None:
		pass

	@property
	def rotation(self) -> Quaternion:
		pass

	@rotation.setter
	def rotation(self, i: Quaternion) -> None:
		pass

	@property
	def scale(self) -> Vector3:
		pass

	@scale.setter
	def scale(self, i: Vector3) -> None:
		pass


#  Bone, skeletal animation bone
class BoneInfo(Structure):
	@property
	def name(self) -> c_char * 32:
		pass

	@name.setter
	def name(self, i: c_char * 32) -> None:
		pass

	@property
	def parent(self) -> int:
		pass

	@parent.setter
	def parent(self, i: int) -> None:
		pass


#  Model, meshes, materials and animation data
class Model(Structure):
	@property
	def transform(self) -> Matrix:
		pass

	@transform.setter
	def transform(self, i: Matrix) -> None:
		pass

	@property
	def meshCount(self) -> int:
		pass

	@meshCount.setter
	def meshCount(self, i: int) -> None:
		pass

	@property
	def materialCount(self) -> int:
		pass

	@materialCount.setter
	def materialCount(self, i: int) -> None:
		pass

	@property
	def meshes(self) -> POINTER(Mesh):
		pass

	@meshes.setter
	def meshes(self, i: POINTER(Mesh)) -> None:
		pass

	@property
	def materials(self) -> POINTER(Material):
		pass

	@materials.setter
	def materials(self, i: POINTER(Material)) -> None:
		pass

	@property
	def meshMaterial(self) -> POINTER(c_int):
		pass

	@meshMaterial.setter
	def meshMaterial(self, i: POINTER(c_int)) -> None:
		pass

	@property
	def boneCount(self) -> int:
		pass

	@boneCount.setter
	def boneCount(self, i: int) -> None:
		pass

	@property
	def bones(self) -> POINTER(BoneInfo):
		pass

	@bones.setter
	def bones(self, i: POINTER(BoneInfo)) -> None:
		pass

	@property
	def bindPose(self) -> POINTER(Transform):
		pass

	@bindPose.setter
	def bindPose(self, i: POINTER(Transform)) -> None:
		pass


#  ModelAnimation
class ModelAnimation(Structure):
	@property
	def boneCount(self) -> int:
		pass

	@boneCount.setter
	def boneCount(self, i: int) -> None:
		pass

	@property
	def frameCount(self) -> int:
		pass

	@frameCount.setter
	def frameCount(self, i: int) -> None:
		pass

	@property
	def bones(self) -> POINTER(BoneInfo):
		pass

	@bones.setter
	def bones(self, i: POINTER(BoneInfo)) -> None:
		pass

	@property
	def framePoses(self) -> POINTER(POINTER(Transform)):
		pass

	@framePoses.setter
	def framePoses(self, i: POINTER(POINTER(Transform))) -> None:
		pass


#  Ray, ray for raycasting
class Ray(Structure):
	@property
	def position(self) -> Vector3:
		pass

	@position.setter
	def position(self, i: Vector3) -> None:
		pass

	@property
	def direction(self) -> Vector3:
		pass

	@direction.setter
	def direction(self, i: Vector3) -> None:
		pass


#  RayCollision, ray hit information
class RayCollision(Structure):
	@property
	def hit(self) -> bool:
		pass

	@hit.setter
	def hit(self, i: bool) -> None:
		pass

	@property
	def distance(self) -> float:
		pass

	@distance.setter
	def distance(self, i: float) -> None:
		pass

	@property
	def point(self) -> Vector3:
		pass

	@point.setter
	def point(self, i: Vector3) -> None:
		pass

	@property
	def normal(self) -> Vector3:
		pass

	@normal.setter
	def normal(self, i: Vector3) -> None:
		pass


#  BoundingBox
class BoundingBox(Structure):
	@property
	def min(self) -> Vector3:
		pass

	@min.setter
	def min(self, i: Vector3) -> None:
		pass

	@property
	def max(self) -> Vector3:
		pass

	@max.setter
	def max(self, i: Vector3) -> None:
		pass


#  VrDeviceInfo, Head-Mounted-Display device parameters
class VrDeviceInfo(Structure):
	@property
	def hResolution(self) -> int:
		pass

	@hResolution.setter
	def hResolution(self, i: int) -> None:
		pass

	@property
	def vResolution(self) -> int:
		pass

	@vResolution.setter
	def vResolution(self, i: int) -> None:
		pass

	@property
	def hScreenSize(self) -> float:
		pass

	@hScreenSize.setter
	def hScreenSize(self, i: float) -> None:
		pass

	@property
	def vScreenSize(self) -> float:
		pass

	@vScreenSize.setter
	def vScreenSize(self, i: float) -> None:
		pass

	@property
	def vScreenCenter(self) -> float:
		pass

	@vScreenCenter.setter
	def vScreenCenter(self, i: float) -> None:
		pass

	@property
	def eyeToScreenDistance(self) -> float:
		pass

	@eyeToScreenDistance.setter
	def eyeToScreenDistance(self, i: float) -> None:
		pass

	@property
	def lensSeparationDistance(self) -> float:
		pass

	@lensSeparationDistance.setter
	def lensSeparationDistance(self, i: float) -> None:
		pass

	@property
	def interpupillaryDistance(self) -> float:
		pass

	@interpupillaryDistance.setter
	def interpupillaryDistance(self, i: float) -> None:
		pass

	@property
	def lensDistortionValues(self) -> c_float * 4:
		pass

	@lensDistortionValues.setter
	def lensDistortionValues(self, i: c_float * 4) -> None:
		pass

	@property
	def chromaAbCorrection(self) -> c_float * 4:
		pass

	@chromaAbCorrection.setter
	def chromaAbCorrection(self, i: c_float * 4) -> None:
		pass


#  VrStereoConfig, VR stereo rendering configuration for simulator
class VrStereoConfig(Structure):
	@property
	def projection(self) -> Matrix * 2:
		pass

	@projection.setter
	def projection(self, i: Matrix * 2) -> None:
		pass

	@property
	def viewOffset(self) -> Matrix * 2:
		pass

	@viewOffset.setter
	def viewOffset(self, i: Matrix * 2) -> None:
		pass

	@property
	def leftLensCenter(self) -> c_float * 2:
		pass

	@leftLensCenter.setter
	def leftLensCenter(self, i: c_float * 2) -> None:
		pass

	@property
	def rightLensCenter(self) -> c_float * 2:
		pass

	@rightLensCenter.setter
	def rightLensCenter(self, i: c_float * 2) -> None:
		pass

	@property
	def leftScreenCenter(self) -> c_float * 2:
		pass

	@leftScreenCenter.setter
	def leftScreenCenter(self, i: c_float * 2) -> None:
		pass

	@property
	def rightScreenCenter(self) -> c_float * 2:
		pass

	@rightScreenCenter.setter
	def rightScreenCenter(self, i: c_float * 2) -> None:
		pass

	@property
	def scale(self) -> c_float * 2:
		pass

	@scale.setter
	def scale(self, i: c_float * 2) -> None:
		pass

	@property
	def scaleIn(self) -> c_float * 2:
		pass

	@scaleIn.setter
	def scaleIn(self, i: c_float * 2) -> None:
		pass


#  File path list
class FilePathList(Structure):
	@property
	def capacity(self) -> int:
		pass

	@capacity.setter
	def capacity(self, i: int) -> None:
		pass

	@property
	def count(self) -> int:
		pass

	@count.setter
	def count(self, i: int) -> None:
		pass

	@property
	def paths(self) -> POINTER(POINTER(c_char)):
		pass

	@paths.setter
	def paths(self, i: POINTER(POINTER(c_char))) -> None:
		pass





from pathlib import Path
import json
import inflection
from raypyc.colors import *
from raypyc.enums import *
from raypyc.structures import *

from ctypes import (
    POINTER,
    c_bool,
    c_char,
    c_char_p,
    c_double,
    c_short,
    c_ubyte,
    c_float,
    c_void_p,
    c_int,
    c_uint,
    c_wchar,
    c_ushort,
    c_long,
    c_ulong,
    c_longlong,
    c_ulonglong,
    c_size_t,
    c_ssize_t,
    c_longdouble,
    c_wchar_p,
    cdll
)

# we don't use spaces...
typesDictionaryCstringToCtypes = {
    'bool': c_bool,  # C type: _Bool  Python type: bool (1)
    'char': c_char,  # C type: char  Python type: 1-character bytes object
    'wchar_t': c_wchar,  # C type: wchar_t  Python type: 1-character string
    # 'char': c_byte,  # C type: char  Python type: int
    'unsignedchar': c_ubyte,  # C type: unsigned char  Python type: int
    'short': c_short,  # C type: short  Python type: int
    'unsignedshort': c_ushort,  # C type: unsigned short  Python type: int
    'int': c_int,  # C type: int  Python type: int
    'unsignedint': c_uint,  # C type: unsigned int  Python type: int
    'long': c_long,  # C type: long  Python type: int
    'unsignedlong': c_ulong,  # C type: unsigned long  Python type: int
    'uint64': c_longlong,  # C type: __int64 or long-long  Python type: int
    'unsigneduint64': c_ulonglong,  # C type: unsigned __int64 or unsigned long-long  Python type: int
    'size_t': c_size_t,  # C type: unsigned size_t  Python type: int
    'ssize_t': c_ssize_t,  # C type: ssize_t or Py_ssize_t  Python type: int
    'float': c_float,  # C type: float  Python type: float
    'double': c_double,  # C type: double  Python type: float
    'longdouble': c_longdouble,  # C type: long double  Python type: float
    'char*': c_char_p,  # C type: char* (NUL terminated)  Python type: bytes object or None
    'wchar_t*': c_wchar_p,  # C type: wchar_t* (NUL terminated)  Python type: string object or None
    'void*': c_void_p,  # C type: wchar_t* (NUL terminated)  Python type: int or None
    'void': None,  # C type: void  Python type: None

    'Vector2': Vector2,
    'Vector3': Vector3,
    'Vector4': Vector4,
    'Quaternion': Quaternion,
    'Matrix': Matrix,
    'Color': Color,
    'Rectangle': Rectangle,
    'Image': Image,
    'Texture': Texture,
    'Texture2D': Texture2D,
    'TextureCubemap': TextureCubemap,
    'RenderTexture': RenderTexture,
    'RenderTexture2D': RenderTexture2D,
    'NPatchInfo': NPatchInfo,
    'GlyphInfo': GlyphInfo,
    'Font': Font,
    'Camera3D': Camera3D,
    'Camera': Camera,
    'Camera2D': Camera2D,
    'Mesh': Mesh,
    'Shader': Shader,
    'MaterialMap': MaterialMap,
    'Material': Material,
    'Transform': Transform,
    'BoneInfo': BoneInfo,
    'Model': Model,
    'ModelAnimation': ModelAnimation,
    'Ray': Ray,
    'RayCollision': RayCollision,
    'BoundingBox': BoundingBox,
    'VrDeviceInfo': VrDeviceInfo,
    'VrStereoConfig': VrStereoConfig,
    'FilePathList': FilePathList
}

typesDictionaryCstringToCtypesString = {
    'bool': 'c_bool',  # C type: _Bool  Python type: bool (1)
    'char': 'c_char',  # C type: char  Python type: 1-character bytes object
    'wchar_t': 'c_wchar',  # C type: wchar_t  Python type: 1-character string
    # 'char': c_byte,  # C type: char  Python type: int
    'unsignedchar': 'c_ubyte',  # C type: unsigned char  Python type: int
    'short': 'c_short',  # C type: short  Python type: int
    'unsignedshort': 'c_ushort',  # C type: unsigned short  Python type: int
    'int': 'c_int',  # C type: int  Python type: int
    'unsignedint': 'c_uint',  # C type: unsigned int  Python type: int
    'long': 'c_long',  # C type: long  Python type: int
    'unsignedlong': 'c_ulong',  # C type: unsigned long  Python type: int
    'uint64': 'c_longlong',  # C type: __int64 or long-long  Python type: int
    'unsigneduint64': 'c_ulonglong',  # C type: unsigned __int64 or unsigned long-long  Python type: int
    'size_t': 'c_size_t',  # C type: unsigned size_t  Python type: int
    'ssize_t': 'c_ssize_t',  # C type: ssize_t or Py_ssize_t  Python type: int
    'float': 'c_float',  # C type: float  Python type: float
    'double': 'c_double',  # C type: double  Python type: float
    'longdouble': 'c_longdouble',  # C type: long double  Python type: float
    'char*': 'c_char_p',  # C type: char* (NUL terminated)  Python type: bytes object or None
    'wchar_t*': 'c_wchar_p',  # C type: wchar_t* (NUL terminated)  Python type: string object or None
    'void*': 'c_void_p',  # C type: wchar_t* (NUL terminated)  Python type: int or None
    'void': 'None',  # C type: void  Python type: None
}

typesDictionaryCstringToPythonTypesString = {
    'bool': 'bool',  # C type: _Bool  Python type: bool (1)
    'char': 'bytes',  # C type: char  Python type: 1-character bytes object
    'wchar_t': 'str',  # C type: wchar_t  Python type: 1-character string
    # 'char': c_byte,  # C type: char  Python type: int
    'unsignedchar': 'int',  # C type: unsigned char  Python type: int
    'short': 'int',  # C type: short  Python type: int
    'unsignedshort': 'int',  # C type: unsigned short  Python type: int
    'int': 'int',  # C type: int  Python type: int
    'unsignedint': 'int',  # C type: unsigned int  Python type: int
    'long': 'int',  # C type: long  Python type: int
    'unsignedlong': 'int',  # C type: unsigned long  Python type: int
    'uint64': 'int',  # C type: __int64 or long-long  Python type: int
    'unsigneduint64': 'int',  # C type: unsigned __int64 or unsigned long-long  Python type: int
    'size_t': 'int',  # C type: unsigned size_t  Python type: int
    'ssize_t': 'int',  # C type: ssize_t or Py_ssize_t  Python type: int
    'float': 'float',  # C type: float  Python type: float
    'double': 'float',  # C type: double  Python type: float
    'longdouble': 'float',  # C type: long double  Python type: float
    'char*': 'bytes',  # C type: char* (NUL terminated)  Python type: bytes object or None
    'wchar_t*': 'string',  # C type: wchar_t* (NUL terminated)  Python type: string object or None
    'void*': 'int',  # C type: wchar_t* (NUL terminated)  Python type: int or None
    'void': 'None',  # C type: void  Python type: None
}

current_module = __import__(__name__)

_rl = cdll.LoadLibrary(str(Path(__file__).parent / 'lib/raylib.dll'))

# get numbers from string
def get_numbers_from_string(string):
    ints = []
    for s in string.split(' '):
        if s.isdigit():
            ints.append(int(s))
    return ints

def wrap_function(funcname, _argtypes, _restype):
    func = _rl.__getattr__(funcname)
    func.argtypes = None if _argtypes == [None] else _argtypes

    func.restype = _restype

    return func


# add function to module
def add_function_to_module(_current_module, name_of_function, function):
    setattr(_current_module, name_of_function, function)


# return a set (from a functions set) of functions that can be wrap
def check_for_functions_that_can_wrap(functions_set):
    functions_that_can_be_wrap = []
    functions_that_cant_be_wrap = []
    # check if function can wrap
    for function in functions_set:
        do_wrapper_this_function = True
        if 'params' in function.keys():
            for function_param in function['params']:
                function_param_type = function_param['type']
                function_param_pointer_level = function_param_type.count("*")
                sfunction_param_is_array = function_param_type.count("]")
                if sfunction_param_is_array == 0:
                    if function_param_pointer_level == 0 or (function_param_type.replace(" ", "").replace("const", "") in typesDictionaryCstringToPythonTypesString):  # if value isn't a pointer value
                        if function_param_type.replace(" ", "").replace('const', '') in typesDictionaryCstringToPythonTypesString:  # there isn't really a const type in python
                            do_wrapper_this_function = True
                        else:  # probably type is struct
                            do_wrapper_this_function = True  # probably type is struct
                    else:
                        if function_param_type.replace('const', '').replace(" ", "") in typesDictionaryCstringToCtypesString:
                            do_wrapper_this_function = True
                        else:
                            if function_param_type.replace('const', '').replace(" ", "").replace("*", "") in typesDictionaryCstringToCtypesString:
                                do_wrapper_this_function = True
                            else:
                                do_wrapper_this_function = True  # probably type is struct
                else:
                    if function_param_type.replace('const', '').replace(" ", "").replace("*", "").replace('[', '').replace(']', '').replace(
                            f'{str(get_numbers_from_string(function_param_type.replace("const", "").replace(" ", "").replace("*", "").replace("[", " ").replace("]", " "))[0])}',
                            '') in typesDictionaryCstringToCtypesString:  # that why in the typesDictionary we don't use spaces for the key
                        do_wrapper_this_function = True
                    else:
                        do_wrapper_this_function = True  # probably type is struct

                if function_param_type.replace('const', '').replace(" ", "").replace("*", "").replace('[', '').replace(']', '') in ['AudioStream', 'Wave', 'Sound', 'Music', 'AudioCallback', 'SaveFileTextCallback', 'LoadFileTextCallback',
                                                                                                                                    'TraceLogCallback', 'LoadFileDataCallback', 'SaveFileDataCallback']:
                    do_wrapper_this_function = False
                    break

        if do_wrapper_this_function:
            function_return_type = function['returnType']
            function_param_pointer_level = function_return_type.count("*")
            sfunction_param_is_array = function_return_type.count("]")
            if sfunction_param_is_array == 0:
                if function_param_pointer_level == 0 or (function_return_type.replace(" ", "").replace('const', '') in typesDictionaryCstringToCtypesString):  # if value isn't a pointer value
                    if function_return_type.replace(" ", "").replace("const", "") in typesDictionaryCstringToCtypesString:  # there isn't really a const type in python
                        do_wrapper_this_function = True
                    else:  # probably type is struct
                        do_wrapper_this_function = True  # probably type is struct
                else:
                    if function_return_type.replace("const", "").replace(" ", "") in typesDictionaryCstringToCtypesString:
                        do_wrapper_this_function = True
                    else:
                        if function_return_type.replace('const', '').replace(" ", "").replace("*", "") in typesDictionaryCstringToCtypesString:
                            do_wrapper_this_function = True
                        else:
                            do_wrapper_this_function = True  # probably type is struct
            else:
                if function_return_type.replace('const', '').replace(" ", "").replace("*", "").replace('[', '').replace(']', '').replace(
                        f'{str(get_numbers_from_string(function_return_type.replace("const", "").replace(" ", "").replace("*", "").replace("[", " ").replace("]", " "))[0])}',
                        '') in typesDictionaryCstringToCtypesString:  # that why in the typesDictionary we don't use spaces for the key
                    do_wrapper_this_function = True
                else:
                    do_wrapper_this_function = True  # probably type is struct

            if function_return_type.replace('const', '').replace(" ", "").replace("*", "").replace('[', '').replace(']', '') in ['AudioStream', 'Wave', 'Sound', 'Music', 'AudioCallback', 'SaveFileTextCallback', 'LoadFileTextCallback',
                                                                                                                                 'TraceLogCallback', 'LoadFileDataCallback', 'SaveFileDataCallback']:
                do_wrapper_this_function = False

        if do_wrapper_this_function:
            functions_that_can_be_wrap.append(function)
        else:
            functions_that_cant_be_wrap.append(function)

    return functions_that_can_be_wrap


# wrap functions to ctypes functions => to _rl ctypes functions
def wrap_functions_to_ctypes_functions_add_function_to_this_module(functions_to_wrap, _current_module):
    for function_to_wrap in functions_to_wrap:
        name_of_function = inflection.underscore(function_to_wrap['name']).replace('3_d', '_3d').replace('2_d', '_2d')
        function_to_wrap_ctype = {'name': function_to_wrap['name'], 'parametersTypes': [], 'returnType': None}

        if 'params' in function_to_wrap.keys():
            for function_param in function_to_wrap['params']:
                function_param_type = function_param['type']
                function_param_pointer_level = function_param_type.count("*")
                # function_param_is_array = function_param_type.count("]") raylib functions shouldn't really return arrays...
                if function_param_pointer_level == 0 or (function_param_type.replace(" ", "").replace('const', '') in typesDictionaryCstringToCtypes):  # if value isn't a pointer value
                    if function_param_type.replace(" ", "").replace('const', '') in typesDictionaryCstringToCtypes:  # there isn't really a const type in python
                        function_to_wrap_ctype['parametersTypes'].append(typesDictionaryCstringToCtypes[function_param_type.replace(" ", "").replace('const', '')])
                else:
                    function_param_processed = function_param_type.replace('const', '').replace(' ', '').replace('*', '')
                    function_param_processed_ctype = None
                    if function_param_processed in typesDictionaryCstringToCtypes:
                        function_param_processed_ctype = typesDictionaryCstringToCtypes[function_param_processed]
                    for i in range(function_param_pointer_level):
                        function_param_processed_ctype = POINTER(function_param_processed_ctype)
                    function_to_wrap_ctype['parametersTypes'].append(function_param_processed_ctype)

        function_return_type = function_to_wrap['returnType']
        function_return_type_pointer_level = function_return_type.count("*")
        # function_param_is_array = function_param_type.count("]") raylib functions shouldn't really return arrays...
        if function_return_type_pointer_level == 0 or (function_return_type.replace(" ", "").replace('const', '') in typesDictionaryCstringToCtypes):  # if value isn't a pointer value
            if function_return_type.replace(" ", "").replace('const', '') in typesDictionaryCstringToCtypes:  # there isn't really a const type in python
                function_to_wrap_ctype['returnType'] = typesDictionaryCstringToCtypes[function_return_type.replace(" ", "").replace('const', '')]
        else:
            function_return_type_processed = function_return_type.replace('const', '').replace(' ', '').replace('*', '')
            function_return_type_processed_ctype = None
            if function_return_type.replace('const', '').replace(' ', '') in typesDictionaryCstringToCtypes:
                function_return_type_processed_ctype = typesDictionaryCstringToCtypes[function_return_type_processed]
            for i in range(function_return_type_pointer_level):
                function_return_type_processed_ctype = POINTER(function_return_type_processed_ctype)
            function_to_wrap_ctype['parametersTypes'].append(function_return_type_processed_ctype)
        f = wrap_function(function_to_wrap_ctype['name'], function_to_wrap_ctype['parametersTypes'], function_to_wrap_ctype['returnType'])
        add_function_to_module(current_module, name_of_function, f)


# load raylib data
with open(Path(Path(__file__).parent / 'raylib_api.json')) as reader:
    raylib_api = json.load(reader)

raylib_api_functions = raylib_api['functions']

functions_to_wrapped = check_for_functions_that_can_wrap(raylib_api_functions)

wrap_functions_to_ctypes_functions_add_function_to_this_module(functions_to_wrapped, current_module)
